@@ -4,11 +4,10 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-"""Wrapper scripte around `llvm-ar`.
+"""Wrapper script around `llvm-ar`.
 """
 
 import sys
 from tools import shared
 
-cmd = [shared.LLVM_AR] + sys.argv[1:]
-sys.exit(shared.run_process(cmd, stdin=sys.stdin, check=False).returncode)
+shared.exec_process([shared.LLVM_AR] + sys.argv[1:])
