@@ -9,17 +9,24 @@ Math.random = () => {
   MAGIC = Math.pow(MAGIC + 1.8912, 3) % 1;
   return MAGIC;
 };
-var TIME = 10000;
-Date.now = () => TIME++;
-if (typeof performance == 'object') performance.now = Date.now;
-if (ENVIRONMENT_IS_NODE) process['hrtime'] = Date.now;
 
-if (!Module) Module = {};
+var TIME = 10000;
+function deterministicNow() {
+  return TIME++;
+}
+
+Date.now = deterministicNow;
+
+// Setting performance.now to deterministicNow doesn't work so we instead
+// use a helper function in parseTools (getPerformanceNow()) to call it
+// directly.
+// if (typeof performance == 'object') performance.now = Date.now;
+
 Module['thisProgram'] = 'thisProgram'; // for consistency between different builds than between runs of the same build
 
 function hashMemory(id) {
   var ret = 0;
-  var len = _sbrk();
+  var len = _sbrk(0);
   for (var i = 0; i < len; i++) {
     ret = (ret*17 + HEAPU8[i])|0;
   }
