@@ -44,13 +44,13 @@ var LibraryEmbind = {
   },
   $FunctionDefinition__deps: ['$createJsInvoker', '$createJsInvokerSignature', '$emittedFunctions'],
   $FunctionDefinition: class {
-    constructor(name, returnType, argumentTypes, functionIndex, thisType = null, isConstructor = false, isAsync = false) {
+    constructor(name, returnType, argumentTypes, functionIndex, thisType = null, isNonnullReturn = false, isAsync = false) {
       this.name = name;
       this.returnType = returnType;
       this.argumentTypes = argumentTypes;
       this.functionIndex = functionIndex;
       this.thisType = thisType;
-      this.isConstructor = isConstructor;
+      this.isNonnullReturn = isNonnullReturn;
       this.isAsync = isAsync;
     }
 
@@ -80,7 +80,7 @@ var LibraryEmbind = {
       // Constructors can return a pointer, but it will be a non-null pointer.
       // Change the return type to the class type so the TS output doesn't
       // have `| null`.
-      if (this.isConstructor && this.returnType instanceof PointerDefinition) {
+      if (this.isNonnullReturn && this.returnType instanceof PointerDefinition) {
         returnType = this.returnType.classType;
       }
       out.push(`): ${nameMap(returnType, true)}`);
@@ -243,18 +243,6 @@ var LibraryEmbind = {
     }
 
   },
-  $printProperty: (prop, nameMap, out) => {
-    const setType = nameMap(prop.type, false);
-    const getType = nameMap(prop.type, true);
-    if (prop.readonly || setType === getType) {
-      out.push(`${prop.readonly ? 'readonly ' : ''}${prop.name}: ${getType}`);
-      return;
-    }
-    // The getter/setter types don't match, so generate each get/set definition.
-    out.push(`get ${prop.name}(): ${getType}`);
-    out.push(`set ${prop.name}(value: ${setType})`);
-  },
-  $ClassProperty__deps: ['$printProperty'],
   $ClassProperty: class {
     constructor(type, name, readonly) {
       this.type = type;
@@ -263,7 +251,15 @@ var LibraryEmbind = {
     }
 
     print(nameMap, out) {
-      printProperty(this, nameMap, out);
+      const setType = nameMap(this.type, false);
+      const getType = nameMap(this.type, true);
+      if (this.readonly || setType === getType) {
+        out.push(`${this.readonly ? 'readonly ' : ''}${this.name}: ${getType}`);
+        return;
+      }
+      // The getter/setter types don't match, so generate each get/set definition.
+      out.push(`get ${this.name}(): ${getType}`);
+      out.push(`set ${this.name}(value: ${setType})`);
     }
   },
   $ConstantDefinition: class {
@@ -329,7 +325,6 @@ var LibraryEmbind = {
       out.push(' ];\n\n');
     }
   },
-  $ValueObjectDefinition__deps: ['$printProperty'],
   $ValueObjectDefinition: class {
     constructor(typeId, name) {
       this.typeId = typeId;
@@ -341,14 +336,12 @@ var LibraryEmbind = {
     }
 
     print(nameMap, out) {
-      out.push(`export type ${this.name} = {\n  `);
+      out.push(`export type ${this.name} = {\n`);
       const outFields = [];
-      for (const field of this.fields) {
-        const property = [];
-        printProperty(field, nameMap, property);
-        outFields.push(...property);
+      for (const {name, type} of this.fields) {
+        outFields.push(`  ${name}: ${nameMap(type)}`);
       }
-      out.push(outFields.join(',\n  '))
+      out.push(outFields.join(',\n'))
       out.push('\n};\n\n');
     }
   },
@@ -463,7 +456,7 @@ var LibraryEmbind = {
     registerType(id, new IntegerType(id));
   },
   $createFunctionDefinition__deps: ['$FunctionDefinition', '$heap32VectorToArray', '$readLatin1String', '$Argument', '$whenDependentTypesAreResolved', '$getFunctionName', '$getFunctionArgsName', '$PointerDefinition', '$ClassDefinition'],
-  $createFunctionDefinition: (name, argCount, rawArgTypesAddr, functionIndex, hasThis, isConstructor, isAsync, cb) => {
+  $createFunctionDefinition: (name, argCount, rawArgTypesAddr, functionIndex, hasThis, isNonnullReturn, isAsync, cb) => {
     const argTypes = heap32VectorToArray(argCount, rawArgTypesAddr);
     name = typeof name === 'string' ? name : readLatin1String(name);
 
@@ -483,8 +476,10 @@ var LibraryEmbind = {
         }
         argStart = 2;
       }
-      if (argsName.length)
-        assert(argsName.length == (argTypes.length - hasThis - 1), 'Argument names should match number of parameters.');
+      if (argsName.length && argsName.length != (argTypes.length - hasThis - 1)) {
+        throw new Error('Argument names should match number of parameters.');
+      }
+
       const args = [];
       for (let i = argStart, x = 0; i < argTypes.length; i++) {
         if (x < argsName.length) {
@@ -493,7 +488,7 @@ var LibraryEmbind = {
           args.push(new Argument(`_${i - argStart}`, argTypes[i]));
         }
       }
-      const funcDef = new FunctionDefinition(name, returnType, args, functionIndex, thisType, isConstructor, isAsync);
+      const funcDef = new FunctionDefinition(name, returnType, args, functionIndex, thisType, isNonnullReturn, isAsync);
       cb(funcDef);
       return [];
     });
@@ -544,8 +539,8 @@ var LibraryEmbind = {
     // TODO
   },
   _embind_register_function__deps: ['$moduleDefinitions', '$createFunctionDefinition'],
-  _embind_register_function: (name, argCount, rawArgTypesAddr, signature, rawInvoker, fn, isAsync) => {
-    createFunctionDefinition(name, argCount, rawArgTypesAddr, fn, false, false, isAsync, (funcDef) => {
+  _embind_register_function: (name, argCount, rawArgTypesAddr, signature, rawInvoker, fn, isAsync, isNonnullReturn) => {
+    createFunctionDefinition(name, argCount, rawArgTypesAddr, fn, false, isNonnullReturn, isAsync, (funcDef) => {
       moduleDefinitions.push(funcDef);
     });
   },
@@ -605,8 +600,9 @@ var LibraryEmbind = {
           rawInvoker,
           context,
           isPureVirtual,
-          isAsync) {
-    createFunctionDefinition(methodName, argCount, rawArgTypesAddr, context, true, false, isAsync, (funcDef) => {
+          isAsync,
+          isNonnullReturn) {
+    createFunctionDefinition(methodName, argCount, rawArgTypesAddr, context, true, isNonnullReturn, isAsync, (funcDef) => {
       const classDef = funcDef.thisType;
       classDef.methods.push(funcDef);
     });
@@ -625,7 +621,10 @@ var LibraryEmbind = {
                                             setterContext) {
     fieldName = readLatin1String(fieldName);
     const readonly = setter === 0;
-    assert(readonly || getterReturnType === setterArgumentType, 'Mismatched getter and setter types are not supported.');
+    if (!(readonly || getterReturnType === setterArgumentType)) {
+      throw new error('Mismatched getter and setter types are not supported.');
+    }
+
     whenDependentTypesAreResolved([], [classType], function(classType) {
       classType = classType[0];
       whenDependentTypesAreResolved([], [getterReturnType], function(types) {
@@ -644,10 +643,11 @@ var LibraryEmbind = {
                                                   invokerSignature,
                                                   rawInvoker,
                                                   fn,
-                                                  isAsync) {
+                                                  isAsync,
+                                                  isNonnullReturn) {
     whenDependentTypesAreResolved([], [rawClassType], function(classType) {
       classType = classType[0];
-      createFunctionDefinition(methodName, argCount, rawArgTypesAddr, fn, false, false, isAsync, (funcDef) => {
+      createFunctionDefinition(methodName, argCount, rawArgTypesAddr, fn, false, isNonnullReturn, isAsync, (funcDef) => {
         classType.staticMethods.push(funcDef);
       });
       return [];
@@ -725,7 +725,10 @@ var LibraryEmbind = {
     setterContext
   ) {
     const valueArray = tupleRegistrations[rawTupleType];
-    assert(getterReturnType === setterArgumentType, 'Mismatched getter and setter types are not supported.');
+    if (getterReturnType !== setterArgumentType) {
+      throw new Error('Mismatched getter and setter types are not supported.');
+    }
+
     valueArray.elementTypeIds.push(getterReturnType);
   },
   _embind_finalize_value_array__deps: ['$whenDependentTypesAreResolved', '$moduleDefinitions', '$tupleRegistrations'],
@@ -766,7 +769,10 @@ var LibraryEmbind = {
     setterContext
   ) {
     const valueObject = structRegistrations[structType];
-    assert(getterReturnType === setterArgumentType, 'Mismatched getter and setter types are not supported.');
+    if (getterReturnType !== setterArgumentType) {
+      throw new Error('Mismatched getter and setter types are not supported.');
+    }
+
     valueObject.fieldTypeIds.push(getterReturnType);
     valueObject.fieldNames.push(readLatin1String(fieldName));
   },
@@ -827,10 +833,10 @@ var LibraryEmbind = {
 #endif
 
   // Stub functions used by eval, but not needed for TS generation:
-  $makeLegalFunctionName: () => assert(false, 'stub function should not be called'),
-  $newFunc: () => assert(false, 'stub function should not be called'),
-  $runDestructors: () => assert(false, 'stub function should not be called'),
-  $createNamedFunction: () => assert(false, 'stub function should not be called'),
+  $makeLegalFunctionName: () => { throw new Error('stub function should not be called'); },
+  $newFunc: () => { throw new Error('stub function should not be called'); },
+  $runDestructors: () => { throw new Error('stub function should not be called'); },
+  $createNamedFunction: () => { throw new Error('stub function should not be called'); },
 };
 
 #if EMBIND_AOT
