@@ -4,8 +4,6 @@
  * SPDX-License-Identifier: MIT
  */
 
-//== HEADLESS ==//
-
 var headlessPrint = (x) => {
   //print(x);
 };
@@ -13,15 +11,18 @@ var headlessPrint = (x) => {
 var window = {
   // adjustable parameters
   location: {
-    toString: function() {
-      return '%s';
+    toString() {
+      return 'http://emscripten.org';
     },
-    search: '?%s',
-    pathname: '%s',
+    search: '',
+    pathname: null,
+  },
+  onIdle() {
+    headlessPrint('triggering click');
+    document.querySelector('.fullscreen-button.low-res').callEventListeners('click');
+    window.onIdle = null;
   },
-  onIdle: function(){ headlessPrint('triggering click'); document.querySelector('.fullscreen-button.low-res').callEventListeners('click'); window.onIdle = null; },
   dirsToDrop: 0, // go back to root dir if first_js is in a subdir
-  //
 
   headless: true,
 
@@ -30,26 +31,26 @@ var window = {
   rafs: [],
   timeouts: [],
   uid: 0,
-  requestAnimationFrame: function(func) {
+  requestAnimationFrame(func) {
     func.uid = window.uid++;
-    headlessPrint('adding raf ' + func.uid);
+    headlessPrint(`adding raf ${func.uid}`);
     window.rafs.push(func);
   },
-  setTimeout: function(func, ms) {
+  setTimeout(func, ms) {
     func.uid = window.uid++;
-    headlessPrint('adding timeout ' + func.uid);
+    headlessPrint(`adding timeout ${func.uid}`);
     window.timeouts.push({
-      func: func,
+      func,
       when: window.fakeNow + (ms || 0)
     });
-    window.timeouts.sort(function(x, y) { return y.when - x.when });
+    window.timeouts.sort((x, y) => { return y.when - x.when });
   },
-  runEventLoop: function() {
+  runEventLoop() {
     // run forever until an exception stops this replay
     var iter = 0;
     while (!this.stopped) {
       var start = Date.realNow();
-      headlessPrint('event loop: ' + (iter++));
+      headlessPrint(`event loop: ${(iter++)}`);
       if (window.rafs.length == 0 && window.timeouts.length == 0) {
         if (window.onIdle) {
           window.onIdle();
@@ -62,7 +63,7 @@ var window = {
       window.rafs = [];
       for (var i = 0; i < currRafs.length; i++) {
         var raf = currRafs[i];
-        headlessPrint('calling raf: ' + raf.uid);// + ': ' + raf.toString().substring(0, 50));
+        headlessPrint(`calling raf: ${raf.uid}`);// + ': ' + raf.toString().substring(0, 50));
         raf();
       }
       // timeouts
@@ -71,23 +72,21 @@ var window = {
       window.timeouts = [];
       while (timeouts.length && timeouts[timeouts.length-1].when <= now) {
         var timeout = timeouts.pop();
-        headlessPrint('calling timeout: ' + timeout.func.uid);// + ': ' + timeout.func.toString().substring(0, 50));
+        headlessPrint(`calling timeout: ${timeout.func.uid}`);// + ': ' + timeout.func.toString().substring(0, 50));
         timeout.func();
       }
       // increment 'time'
       window.fakeNow += 16.666;
-      headlessPrint('main event loop iteration took ' + (Date.realNow() - start) + ' ms');
+      headlessPrint(`main event loop iteration took ${Date.realNow() - start} ms`);
     }
   },
   eventListeners: {},
-  addEventListener: function(id, func) {
+  addEventListener(id, func) {
     var listeners = this.eventListeners[id];
-    if (!listeners) {
-      listeners = this.eventListeners[id] = [];
-    }
+    listeners ||= this.eventListeners[id] = [];
     listeners.push(func);
   },
-  removeEventListener: function(id, func) {
+  removeEventListener(id, func) {
     var listeners = this.eventListeners[id];
     if (!listeners) return;
     for (var i = 0; i < listeners.length; i++) {
@@ -97,19 +96,17 @@ var window = {
       }
     }
   },
-  callEventListeners: function(id) {
+  callEventListeners(id) {
     var listeners = this.eventListeners[id];
-    if (listeners) {
-      listeners.forEach(function(listener) { listener() });
-    }
+    listeners?.forEach((listener) => listener());
   },
   URL: {
-    createObjectURL: function(x) {
+    createObjectURL(x) {
       return x; // the blob itself is returned
     },
-    revokeObjectURL: function(x) {},
+    revokeObjectURL(x) {},
   },
-  encodeURIComponent: function(x) { return x },
+  encodeURIComponent(x) { return x },
 };
 var setTimeout = window.setTimeout;
 var document = {
@@ -118,7 +115,7 @@ var document = {
   addEventListener: window.addEventListener,
   removeEventListener: window.removeEventListener,
   callEventListeners: window.callEventListeners,
-  getElementById: function(id) {
+  getElementById(id) {
     switch (id) {
       case 'canvas': {
         if (this.canvas) return this.canvas;
@@ -130,17 +127,17 @@ var document = {
       default: throw 'getElementById: ' + id;
     }
   },
-  createElement: function(what) {
+  createElement(what) {
     switch (what) {
       case 'canvas': return document.getElementById(what);
       case 'script': {
         var ret = {};
-        window.setTimeout(function() {
-          headlessPrint('loading script: ' + ret.src);
+        window.setTimeout(() => {
+          headlessPrint(`loading script: ${ret.src}`);
           load(ret.src);
           headlessPrint('   script loaded.');
           if (ret.onload) {
-            window.setTimeout(function() {
+            window.setTimeout(() => {
               ret.onload(); // yeah yeah this might vanish
             });
           }
@@ -149,46 +146,44 @@ var document = {
       }
       case 'div': {
         return {
-          appendChild: function() {},
-          requestFullscreen: function() {
+          appendChild() {},
+          requestFullscreen() {
             return document.getElementById('canvas').requestFullscreen();
           },
         };
       }
-      default: throw 'createElement ' + what + new Error().stack;
+      default: throw `createElement ${what}${new Error().stack}`;
     }
   },
   elements: {},
-  querySelector: function(id) {
-    if (!document.elements[id]) {
-      document.elements[id] = {
-        classList: {
-          add: function(){},
-          remove: function(){},
-        },
-        eventListeners: {},
-        addEventListener: document.addEventListener,
-        removeEventListener: document.removeEventListener,
-        callEventListeners: document.callEventListeners,
-      };
+  querySelector(id) {
+    document.elements[id] ||= {
+      classList: {
+        add() {},
+        remove() {},
+      },
+      eventListeners: {},
+      addEventListener: document.addEventListener,
+      removeEventListener: document.removeEventListener,
+      callEventListeners: document.callEventListeners,
     };
     return document.elements[id];
   },
   styleSheets: [{
     cssRules: [],
-    insertRule: function(){},
+    insertRule() {},
   }],
   body: {
-    appendChild: function(){},
+    appendChild() {},
   },
-  exitPointerLock: function(){},
-  exitFullscreen: function(){},
+  exitPointerLock() {},
+  exitFullscreen() {},
 };
 var alert = function(x) {
   print(x);
 };
 var performance = {
-  now: function() {
+  now() {
     return Date.now();
   },
 };
@@ -201,23 +196,23 @@ function fixPath(path) {
 }
 var XMLHttpRequest = function() {
   return {
-    open: function(mode, path, async) {
+    open(mode, path, async) {
       path = fixPath(path);
       this.mode = mode;
       this.path = path;
       this.async = async;
     },
-    send: function() {
+    send() {
       if (!this.async) {
         this.doSend();
       } else {
         window.setTimeout(() => {
           this.doSend();
-          if (this.onload) this.onload();
+          this.onload?.();
         }, 0);
       }
     },
-    doSend: function() {
+    doSend() {
       if (this.responseType == 'arraybuffer') {
         this.response = read(this.path, 'binary');
       } else {
@@ -226,34 +221,32 @@ var XMLHttpRequest = function() {
     },
   };
 };
-var Audio = () => {
-  return {
-    play: function(){},
-    pause: function(){},
-    cloneNode: function() {
-      return this;
-    },
-  };
-};
+var Audio = () => ({
+  play() {},
+  pause() {},
+  cloneNode() {
+    return this;
+  },
+});
 var Image = () => {
   window.setTimeout(() => {
     this.complete = true;
     this.width = 64;
     this.height = 64;
-    if (this.onload) this.onload();
+    this.onload?.();
   });
 };
 var Worker = (workerPath) => {
   workerPath = fixPath(workerPath);
   var workerCode = read(workerPath);
   workerCode = workerCode.replace(/Module/g, 'zzModuleyy' + (Worker.id++)). // prevent collision with the global Module object. Note that this becomes global, so we need unique ids
-                          replace(/\nonmessage = /, '\nvar onmessage = '); // workers commonly do "onmessage = ", we need to varify that to sandbox
-  headlessPrint('loading worker ' + workerPath + ' : ' + workerCode.substring(0, 50));
+                          replace(/\nonmessage = /, '\nvar onmessage = '); // workers commonly do "onmessage = ", we need to verify that to sandbox
+  headlessPrint(`loading worker ${workerPath} : ${workerCode.substring(0, 50)}`);
   eval(workerCode); // will implement onmessage()
 
   function duplicateJSON(json) {
     function handleTypedArrays(key, value) {
-      if (value && value.toString && value.toString().substring(0, 8) == '[object ' && value.length && value.byteLength) {
+      if (value?.toString && value.toString().substring(0, 8) == '[object ' && value.length && value.byteLength) {
         return Array.prototype.slice.call(value);
       }
       return value;
@@ -263,18 +256,18 @@ var Worker = (workerPath) => {
   this.terminate = () => {};
   this.postMessage = (msg) => {
     msg.messageId = Worker.messageId++;
-    headlessPrint('main thread sending message ' + msg.messageId + ' to worker ' + workerPath);
-    window.setTimeout(function() {
-      headlessPrint('worker ' + workerPath + ' receiving message ' + msg.messageId);
+    headlessPrint(`main thread sending message ${msg.messageId} to worker ${workerPath}`);
+    window.setTimeout(() => {
+      headlessPrint(`worker ${workerPath} receiving message ${msg.messageId}`);
       onmessage({ data: duplicateJSON(msg) });
     });
   };
   var thisWorker = this;
   var postMessage = (msg) => {
     msg.messageId = Worker.messageId++;
-    headlessPrint('worker ' + workerPath + ' sending message ' + msg.messageId);
-    window.setTimeout(function() {
-      headlessPrint('main thread receiving message ' + msg.messageId + ' from ' + workerPath);
+    headlessPrint(`worker ${workerPath} sending message ${msg.messageId}`);
+    window.setTimeout(() => {
+      headlessPrint(`main thread receiving message ${msg.messageId} from ${workerPath}`);
       thisWorker.onmessage({ data: duplicateJSON(msg) });
     });
   };
@@ -287,31 +280,11 @@ var screen = { // XXX these values may need to be adjusted
   availWidth: 2100,
   availHeight: 1283,
 };
-if (typeof console == "undefined") {
+if (typeof console == 'undefined') {
   console = {
-    log: function(x) {
-      print(x);
-    }
+    log(x) { print(x); },
   };
 }
-var MozBlobBuilder = () => {
-  this.data = new Uint8Array(0);
-  this.append = function(buffer) {
-    var data = new Uint8Array(buffer);
-    var combined = new Uint8Array(this.data.length + data.length);
-    combined.set(this.data);
-    combined.set(data, this.data.length);
-    this.data = combined;
-  };
-  this.getBlob = function() {
-    return this.data.buffer; // return the buffer as a "blob". XXX We might need to change this if it is not opaque
-  };
-};
 
 // additional setup
-if (!Module['canvas']) {
-  Module['canvas'] = document.getElementById('canvas');
-}
-
-//== HEADLESS ==//
-
+Module['canvas'] ||= document.getElementById('canvas');
