@@ -8,10 +8,10 @@
 // Async support via ASYNCIFY
 //
 
-mergeInto(LibraryManager.library, {
+addToLibrary({
   // error handling
 
-  $runAndAbortIfError: function(func) {
+  $runAndAbortIfError: (func) => {
     try {
       return func();
     } catch (e) {
@@ -20,12 +20,148 @@ mergeInto(LibraryManager.library, {
   },
 
 #if ASYNCIFY
-  $Asyncify__deps: ['$runAndAbortIfError', '$callUserCallback',
+  $Asyncify__deps: ['$runAndAbortIfError', '$callUserCallback', '$sigToWasmTypes',
 #if !MINIMAL_RUNTIME
-    '$runtimeKeepalivePush', '$runtimeKeepalivePop'
+    '$runtimeKeepalivePush', '$runtimeKeepalivePop',
+#endif
+#if ASYNCIFY == 1
+    // Needed by allocateData and handleSleep respectively
+    'malloc', 'free',
 #endif
   ],
+
   $Asyncify: {
+    //
+    // Asyncify code that is shared between mode 1 (original) and mode 2 (JSPI).
+    //
+#if ASYNCIFY == 1 && MEMORY64
+    rewindArguments: {},
+#endif
+    instrumentWasmImports(imports) {
+#if ASYNCIFY_DEBUG
+      dbg('asyncify instrumenting imports');
+#endif
+#if ASSERTIONS && ASYNCIFY == 2
+      assert('Suspending' in WebAssembly, 'JSPI not supported by current environment. Perhaps it needs to be enabled via flags?');
+#endif
+      var importPattern = {{{ new RegExp(`^(${ASYNCIFY_IMPORTS_EXCEPT_JS_LIBS.map(x => x.split('.')[1]).join('|').replace(/\*/g, '.*')})$`) }}};
+
+      for (let [x, original] of Object.entries(imports)) {
+        if (typeof original == 'function') {
+          let isAsyncifyImport = original.isAsync || importPattern.test(x);
+#if ASYNCIFY == 2
+          // Wrap async imports with a suspending WebAssembly function.
+          if (isAsyncifyImport) {
+#if ASYNCIFY_DEBUG
+            dbg('asyncify: suspendOnReturnedPromise for', x, original);
+#endif
+            imports[x] = original = new WebAssembly.Suspending(original);
+          }
+#endif
+#if ASSERTIONS && ASYNCIFY != 2 // We cannot apply assertions with stack switching, as the imports must not be modified from suspender.suspendOnReturnedPromise TODO find a way
+          imports[x] = (...args) => {
+            var originalAsyncifyState = Asyncify.state;
+            try {
+              return original(...args);
+            } finally {
+              // Only asyncify-declared imports are allowed to change the
+              // state.
+              // Changing the state from normal to disabled is allowed (in any
+              // function) as that is what shutdown does (and we don't have an
+              // explicit list of shutdown imports).
+              var changedToDisabled =
+                    originalAsyncifyState === Asyncify.State.Normal &&
+                    Asyncify.state        === Asyncify.State.Disabled;
+              // invoke_* functions are allowed to change the state if we do
+              // not ignore indirect calls.
+              var ignoredInvoke = x.startsWith('invoke_') &&
+                                  {{{ !ASYNCIFY_IGNORE_INDIRECT }}};
+              if (Asyncify.state !== originalAsyncifyState &&
+                  !isAsyncifyImport &&
+                  !changedToDisabled &&
+                  !ignoredInvoke) {
+                throw new Error(`import ${x} was not in ASYNCIFY_IMPORTS, but changed the state`);
+              }
+            }
+          };
+#if MAIN_MODULE
+          // The dynamic library loader needs to be able to read .sig
+          // properties, so that it knows function signatures when it adds
+          // them to the table.
+          imports[x].sig = original.sig;
+#endif // MAIN_MODULE
+#endif // ASSERTIONS
+        }
+      }
+    },
+#if ASYNCIFY == 1 && MEMORY64
+    saveRewindArguments(funcName, passedArguments) {
+      return Asyncify.rewindArguments[funcName] = Array.from(passedArguments)
+    },
+    restoreRewindArguments(funcName) {
+      return Asyncify.rewindArguments[funcName] || []
+    },
+#endif
+    instrumentWasmExports(exports) {
+#if ASYNCIFY_DEBUG
+      dbg('asyncify instrumenting exports');
+#endif
+#if ASYNCIFY == 2
+      var exportPattern = {{{ new RegExp(`^(${ASYNCIFY_EXPORTS.join('|').replace(/\*/g, '.*')})$`) }}};
+      Asyncify.asyncExports = new Set();
+#endif
+      var ret = {};
+      for (let [x, original] of Object.entries(exports)) {
+        if (typeof original == 'function') {
+#if ASYNCIFY == 2
+          // Wrap all exports with a promising WebAssembly function.
+          let isAsyncifyExport = exportPattern.test(x);
+          if (isAsyncifyExport) {
+            Asyncify.asyncExports.add(original);
+            original = Asyncify.makeAsyncFunction(original);
+          }
+#endif
+          ret[x] = (...args) => {
+#if ASYNCIFY_DEBUG >= 2
+            dbg(`ASYNCIFY: ${'  '.repeat(Asyncify.exportCallStack.length} try ${x}`);
+#endif
+#if ASYNCIFY == 1
+            Asyncify.exportCallStack.push(x);
+            try {
+#endif
+#if ASYNCIFY == 1 && MEMORY64
+              Asyncify.saveRewindArguments(x, args);
+#endif
+              return original(...args);
+#if ASYNCIFY == 1
+            } finally {
+              if (!ABORT) {
+                var y = Asyncify.exportCallStack.pop();
+#if ASSERTIONS
+                assert(y === x);
+#endif
+#if ASYNCIFY_DEBUG >= 2
+                dbg(`ASYNCIFY: ${'  '.repeat(Asyncify.exportCallStack.length)} finally ${x}`);
+#endif
+                Asyncify.maybeStopUnwind();
+              }
+            }
+#endif
+          };
+#if MAIN_MODULE
+          ret[x].orig = original;
+#endif
+        } else {
+          ret[x] = original;
+        }
+      }
+      return ret;
+    },
+
+#if ASYNCIFY == 1
+    //
+    // Original implementation of Asyncify.
+    //
     State: {
       Normal: 0,
       Unwinding: 1,
@@ -36,7 +172,7 @@ mergeInto(LibraryManager.library, {
     StackSize: {{{ ASYNCIFY_STACK_SIZE }}},
     currData: null,
     // The return value passed to wakeUp() in
-    // Asyncify.handleSleep(function(wakeUp){...}) is stored here,
+    // Asyncify.handleSleep((wakeUp) => {...}) is stored here,
     // so we can return it later from the C function that called
     // Asyncify.handleSleep() after rewinding finishes.
     handleSleepReturnValue: 0,
@@ -50,7 +186,7 @@ mergeInto(LibraryManager.library, {
     asyncPromiseHandlers: null, // { resolve, reject } pair for when *all* asynchronicity is done
     sleepCallbacks: [], // functions to call every time we sleep
 
-    getCallStackId: function(funcName) {
+    getCallStackId(funcName) {
       var id = Asyncify.callStackNameToId[funcName];
       if (id === undefined) {
         id = Asyncify.callStackId++;
@@ -60,119 +196,42 @@ mergeInto(LibraryManager.library, {
       return id;
     },
 
-#if ASSERTIONS
-    instrumentWasmImports: function(imports) {
-      var ASYNCIFY_IMPORTS = {{{ JSON.stringify(ASYNCIFY_IMPORTS) }}}.map((x) => x.split('.')[1]);
-      for (var x in imports) {
-        (function(x) {
-          var original = imports[x];
-          if (typeof original == 'function') {
-            imports[x] = function() {
-              var originalAsyncifyState = Asyncify.state;
-              try {
-                return original.apply(null, arguments);
-              } finally {
-                // Only asyncify-declared imports are allowed to change the
-                // state.
-                var isAsyncifyImport = ASYNCIFY_IMPORTS.indexOf(x) >= 0 ||
-                                       x.startsWith('__asyncjs__');
-                // Changing the state from normal to disabled is allowed (in any
-                // function) as that is what shutdown does (and we don't have an
-                // explicit list of shutdown imports).
-                var changedToDisabled =
-                      originalAsyncifyState === Asyncify.State.Normal &&
-                      Asyncify.state        === Asyncify.State.Disabled;
-                // invoke_* functions are allowed to change the state if we do
-                // not ignore indirect calls.
-                var ignoredInvoke = x.startsWith('invoke_') &&
-                                    {{{ !ASYNCIFY_IGNORE_INDIRECT }}};
-                if (Asyncify.state !== originalAsyncifyState &&
-                    !isAsyncifyImport &&
-                    !changedToDisabled &&
-                    !ignoredInvoke) {
-                  throw new Error('import ' + x + ' was not in ASYNCIFY_IMPORTS, but changed the state');
-                }
-              }
-            }
-#if MAIN_MODULE
-            // The dynamic library loader needs to be able to read .sig
-            // properties, so that it knows function signatures when it adds
-            // them to the table.
-            imports[x].sig = original.sig;
-#endif
-          }
-        })(x);
-      }
-    },
-#endif
-
-    instrumentWasmExports: function(exports) {
-      var ret = {};
-      for (var x in exports) {
-        (function(x) {
-          var original = exports[x];
-          if (typeof original == 'function') {
-            ret[x] = function() {
-#if ASYNCIFY_DEBUG >= 2
-              err('ASYNCIFY: ' + '  '.repeat(Asyncify.exportCallStack.length) + ' try ' + x);
-#endif
-              Asyncify.exportCallStack.push(x);
-              try {
-                return original.apply(null, arguments);
-              } finally {
-                if (!ABORT) {
-                  var y = Asyncify.exportCallStack.pop();
-                  assert(y === x);
-#if ASYNCIFY_DEBUG >= 2
-                  err('ASYNCIFY: ' + '  '.repeat(Asyncify.exportCallStack.length) + ' finally ' + x);
-#endif
-                  Asyncify.maybeStopUnwind();
-                }
-              }
-            };
-          } else {
-            ret[x] = original;
-          }
-        })(x);
-      }
-      return ret;
-    },
-
-    maybeStopUnwind: function() {
+    maybeStopUnwind() {
 #if ASYNCIFY_DEBUG
-      err('ASYNCIFY: maybe stop unwind', Asyncify.exportCallStack);
+      dbg('ASYNCIFY: maybe stop unwind', Asyncify.exportCallStack);
 #endif
       if (Asyncify.currData &&
           Asyncify.state === Asyncify.State.Unwinding &&
           Asyncify.exportCallStack.length === 0) {
         // We just finished unwinding.
+        // Be sure to set the state before calling any other functions to avoid
+        // possible infinite recursion here (For example in debug pthread builds
+        // the dbg() function itself can call back into WebAssembly to get the
+        // current pthread_self() pointer).
+        Asyncify.state = Asyncify.State.Normal;
 #if ASYNCIFY_DEBUG
-        err('ASYNCIFY: stop unwind');
+        dbg('ASYNCIFY: stop unwind');
 #endif
         {{{ runtimeKeepalivePush(); }}}
-        Asyncify.state = Asyncify.State.Normal;
         // Keep the runtime alive so that a re-wind can be done later.
-        runAndAbortIfError(Module['_asyncify_stop_unwind']);
+        runAndAbortIfError(_asyncify_stop_unwind);
         if (typeof Fibers != 'undefined') {
           Fibers.trampoline();
         }
       }
     },
 
-    whenDone: function() {
+    whenDone() {
 #if ASSERTIONS
       assert(Asyncify.currData, 'Tried to wait for an async operation when none is in progress.');
       assert(!Asyncify.asyncPromiseHandlers, 'Cannot have multiple async operations in flight at once');
 #endif
       return new Promise((resolve, reject) => {
-        Asyncify.asyncPromiseHandlers = {
-          resolve: resolve,
-          reject: reject
-        };
+        Asyncify.asyncPromiseHandlers = { resolve, reject };
       });
     },
 
-    allocateData: function() {
+    allocateData() {
       // An asyncify data structure has three fields:
       //  0  current stack pos
       //  4  max stack pos
@@ -187,45 +246,71 @@ mergeInto(LibraryManager.library, {
       return ptr;
     },
 
-    setDataHeader: function(ptr, stack, stackSize) {
-      {{{ makeSetValue('ptr', C_STRUCTS.asyncify_data_s.stack_ptr, 'stack', 'i32') }}};
-      {{{ makeSetValue('ptr', C_STRUCTS.asyncify_data_s.stack_limit, 'stack + stackSize', 'i32') }}};
+    setDataHeader(ptr, stack, stackSize) {
+      {{{ makeSetValue('ptr', C_STRUCTS.asyncify_data_s.stack_ptr, 'stack', '*') }}};
+      {{{ makeSetValue('ptr', C_STRUCTS.asyncify_data_s.stack_limit, 'stack + stackSize', '*') }}};
     },
 
-    setDataRewindFunc: function(ptr) {
+    setDataRewindFunc(ptr) {
       var bottomOfCallStack = Asyncify.exportCallStack[0];
 #if ASYNCIFY_DEBUG >= 2
-      err('ASYNCIFY: setDataRewindFunc('+ptr+'), bottomOfCallStack is', bottomOfCallStack, new Error().stack);
+      dbg('ASYNCIFY: setDataRewindFunc('+ptr+'), bottomOfCallStack is', bottomOfCallStack, new Error().stack);
 #endif
       var rewindId = Asyncify.getCallStackId(bottomOfCallStack);
       {{{ makeSetValue('ptr', C_STRUCTS.asyncify_data_s.rewind_id, 'rewindId', 'i32') }}};
     },
 
-    getDataRewindFunc: function(ptr) {
+    getDataRewindFuncName(ptr) {
       var id = {{{ makeGetValue('ptr', C_STRUCTS.asyncify_data_s.rewind_id, 'i32') }}};
       var name = Asyncify.callStackIdToName[id];
-      var func = Module['asm'][name];
+      return name;
+    },
+
+#if RELOCATABLE
+    getDataRewindFunc__deps: [ '$resolveGlobalSymbol' ],
+#endif
+    getDataRewindFunc(name) {
+      var func = wasmExports[name];
+#if RELOCATABLE
+      // Exported functions in side modules are not listed in `wasmExports`,
+      // So we should use `resolveGlobalSymbol` helper function, which is defined in `library_dylink.js`.
+      if (!func) {
+        func = resolveGlobalSymbol(name, false).sym;
+      }
+#endif
       return func;
     },
 
-    doRewind: function(ptr) {
-      var start = Asyncify.getDataRewindFunc(ptr);
+    doRewind(ptr) {
+      var name = Asyncify.getDataRewindFuncName(ptr);
+      var func = Asyncify.getDataRewindFunc(name);
 #if ASYNCIFY_DEBUG
-      err('ASYNCIFY: start:', start);
+      dbg('ASYNCIFY: doRewind:', name);
 #endif
-      // Once we have rewound and the stack we no longer need to artificially keep
-      // the runtime alive.
+      // Once we have rewound and the stack we no longer need to artificially
+      // keep the runtime alive.
       {{{ runtimeKeepalivePop(); }}}
-      return start();
+#if MEMORY64
+      // When re-winding, the arguments to a function are ignored.  For i32 arguments we
+      // can just call the function with no args at all since and the engine will produce zeros
+      // for all arguments.  However, for i64 arguments we get `undefined cannot be converted to
+      // BigInt`.
+      return func(...Asyncify.restoreRewindArguments(name));
+#else
+      return func();
+#endif
     },
 
-    handleSleep: function(startAsync) {
+    // This receives a function to call to start the async operation, and
+    // handles everything else for the user of this API. See emscripten_sleep()
+    // and other async methods for simple examples of usage.
+    handleSleep(startAsync) {
 #if ASSERTIONS
       assert(Asyncify.state !== Asyncify.State.Disabled, 'Asyncify cannot be done during or after the runtime exits');
 #endif
       if (ABORT) return;
 #if ASYNCIFY_DEBUG
-      err('ASYNCIFY: handleSleep ' + Asyncify.state);
+      dbg(`ASYNCIFY: handleSleep ${Asyncify.state}`);
 #endif
       if (Asyncify.state === Asyncify.State.Normal) {
         // Prepare to sleep. Call startAsync, and see what happens:
@@ -234,12 +319,12 @@ mergeInto(LibraryManager.library, {
         // need to do anything.
         var reachedCallback = false;
         var reachedAfterCallback = false;
-        startAsync((handleSleepReturnValue) => {
+        startAsync((handleSleepReturnValue = 0) => {
 #if ASSERTIONS
           assert(!handleSleepReturnValue || typeof handleSleepReturnValue == 'number' || typeof handleSleepReturnValue == 'boolean'); // old emterpretify API supported other stuff
 #endif
           if (ABORT) return;
-          Asyncify.handleSleepReturnValue = handleSleepReturnValue || 0;
+          Asyncify.handleSleepReturnValue = handleSleepReturnValue;
           reachedCallback = true;
           if (!reachedAfterCallback) {
             // We are happening synchronously, so no need for async.
@@ -254,12 +339,12 @@ mergeInto(LibraryManager.library, {
           assert(!Asyncify.exportCallStack.length, 'Waking up (starting to rewind) must be done from JS, without compiled code on the stack.');
 #endif
 #if ASYNCIFY_DEBUG
-          err('ASYNCIFY: start rewind ' + Asyncify.currData);
+          dbg(`ASYNCIFY: start rewind ${Asyncify.currData}`);
 #endif
           Asyncify.state = Asyncify.State.Rewinding;
-          runAndAbortIfError(() => Module['_asyncify_start_rewind'](Asyncify.currData));
-          if (typeof Browser != 'undefined' && Browser.mainLoop.func) {
-            Browser.mainLoop.resume();
+          runAndAbortIfError(() => _asyncify_start_rewind(Asyncify.currData));
+          if (typeof MainLoop != 'undefined' && MainLoop.func) {
+            MainLoop.resume();
           }
           var asyncWasmReturnValue, isError = false;
           try {
@@ -304,26 +389,26 @@ mergeInto(LibraryManager.library, {
           // TODO: reuse, don't alloc/free every sleep
           Asyncify.currData = Asyncify.allocateData();
 #if ASYNCIFY_DEBUG
-          err('ASYNCIFY: start unwind ' + Asyncify.currData);
+          dbg(`ASYNCIFY: start unwind ${Asyncify.currData}`);
 #endif
-          runAndAbortIfError(() => Module['_asyncify_start_unwind'](Asyncify.currData));
-          if (typeof Browser != 'undefined' && Browser.mainLoop.func) {
-            Browser.mainLoop.pause();
+          if (typeof MainLoop != 'undefined' && MainLoop.func) {
+            MainLoop.pause();
           }
+          runAndAbortIfError(() => _asyncify_start_unwind(Asyncify.currData));
         }
       } else if (Asyncify.state === Asyncify.State.Rewinding) {
         // Stop a resume.
 #if ASYNCIFY_DEBUG
-        err('ASYNCIFY: stop rewind');
+        dbg('ASYNCIFY: stop rewind');
 #endif
         Asyncify.state = Asyncify.State.Normal;
-        runAndAbortIfError(Module['_asyncify_stop_rewind']);
+        runAndAbortIfError(_asyncify_stop_rewind);
         _free(Asyncify.currData);
         Asyncify.currData = null;
         // Call all sleep callbacks now that the sleep-resume is all done.
-        Asyncify.sleepCallbacks.forEach((func) => callUserCallback(func));
+        Asyncify.sleepCallbacks.forEach(callUserCallback);
       } else {
-        abort('invalid state: ' + Asyncify.state);
+        abort(`invalid state: ${Asyncify.state}`);
       }
       return Asyncify.handleSleepReturnValue;
     },
@@ -333,48 +418,63 @@ mergeInto(LibraryManager.library, {
     //
     // This is particularly useful for native JS `async` functions where the
     // returned value will "just work" and be passed back to C++.
-    handleAsync: function(startAsync) {
+    handleAsync(startAsync) {
       return Asyncify.handleSleep((wakeUp) => {
         // TODO: add error handling as a second param when handleSleep implements it.
         startAsync().then(wakeUp);
       });
     },
+
+#elif ASYNCIFY == 2
+    //
+    // JSPI implementation of Asyncify.
+    //
+
+    // Stores all the exported raw Wasm functions that are wrapped with async
+    // WebAssembly.Functions.
+    asyncExports: null,
+    isAsyncExport(func) {
+      return Asyncify.asyncExports?.has(func);
+    },
+    handleAsync: async (startAsync) => {
+      {{{ runtimeKeepalivePush(); }}}
+      try {
+        return await startAsync();
+      } finally {
+        {{{ runtimeKeepalivePop(); }}}
+      }
+    },
+    handleSleep(startAsync) {
+      return Asyncify.handleAsync(() => new Promise(startAsync));
+    },
+    makeAsyncFunction(original) {
+#if ASYNCIFY_DEBUG
+      dbg('asyncify: makeAsyncFunction for', original);
+#endif
+      return WebAssembly.promising(original);
+    },
+#endif
   },
 
   emscripten_sleep__deps: ['$safeSetTimeout'],
-  emscripten_sleep: function(ms) {
-    Asyncify.handleSleep((wakeUp) => safeSetTimeout(wakeUp, ms));
-  },
-
-  emscripten_wget__deps: ['$Browser', '$PATH_FS', '$FS'],
-  emscripten_wget: function(url, file) {
-    Asyncify.handleSleep((wakeUp) => {
-      var _url = UTF8ToString(url);
-      var _file = UTF8ToString(file);
-      _file = PATH_FS.resolve(FS.cwd(), _file);
-      var destinationDirectory = PATH.dirname(_file);
-      FS.createPreloadedFile(
-        destinationDirectory,
-        PATH.basename(_file),
-        _url, true, true,
-        wakeUp,
-        wakeUp,
-        false, // dontCreateFile
-        false, // canOwn
-        // preFinish: if the destination directory does not yet exist, create it
-        () => FS.mkdirTree(destinationDirectory)
-      );
-    });
+  emscripten_sleep__async: true,
+  emscripten_sleep: (ms) => {
+    // emscripten_sleep() does not return a value, but we still need a |return|
+    // here for stack switching support (ASYNCIFY=2). In that mode this function
+    // returns a Promise instead of nothing, and that Promise is what tells the
+    // wasm VM to pause the stack.
+    return Asyncify.handleSleep((wakeUp) => safeSetTimeout(wakeUp, ms));
   },
 
   emscripten_wget_data__deps: ['$asyncLoad', 'malloc'],
-  emscripten_wget_data: function(url, pbuffer, pnum, perror) {
-    Asyncify.handleSleep((wakeUp) => {
+  emscripten_wget_data__async: true,
+  emscripten_wget_data: (url, pbuffer, pnum, perror) => {
+    return Asyncify.handleSleep((wakeUp) => {
       asyncLoad(UTF8ToString(url), (byteArray) => {
         // can only allocate the buffer after the wakeUp, not during an asyncing
         var buffer = _malloc(byteArray.length); // must be freed by caller!
         HEAPU8.set(byteArray, buffer);
-        {{{ makeSetValue('pbuffer', 0, 'buffer', 'i32') }}};
+        {{{ makeSetValue('pbuffer', 0, 'buffer', '*') }}};
         {{{ makeSetValue('pnum',  0, 'byteArray.length', 'i32') }}};
         {{{ makeSetValue('perror',  0, '0', 'i32') }}};
         wakeUp();
@@ -386,44 +486,56 @@ mergeInto(LibraryManager.library, {
   },
 
   emscripten_scan_registers__deps: ['$safeSetTimeout'],
-  emscripten_scan_registers: function(func) {
-    Asyncify.handleSleep((wakeUp) => {
+  emscripten_scan_registers__async: true,
+  emscripten_scan_registers: (func) => {
+    return Asyncify.handleSleep((wakeUp) => {
       // We must first unwind, so things are spilled to the stack. Then while
       // we are pausing we do the actual scan. After that we can resume. Note
       // how using a timeout here avoids unbounded call stack growth, which
       // could happen if we tried to scan the stack immediately after unwinding.
       safeSetTimeout(() => {
         var stackBegin = Asyncify.currData + {{{ C_STRUCTS.asyncify_data_s.__size__ }}};
-        var stackEnd = HEAP32[Asyncify.currData >> 2];
-        {{{ makeDynCall('vii', 'func') }}}(stackBegin, stackEnd);
+        var stackEnd = {{{ makeGetValue('Asyncify.currData', 0, '*') }}};
+        {{{ makeDynCall('vpp', 'func') }}}(stackBegin, stackEnd);
         wakeUp();
       }, 0);
     });
   },
 
-  emscripten_lazy_load_code: function() {
-    Asyncify.handleSleep((wakeUp) => {
-      // Update the expected wasm binary file to be the lazy one.
-      wasmBinaryFile += '.lazy.wasm';
-      // Add a callback for when all run dependencies are fulfilled, which happens when async wasm loading is done.
-      dependenciesFulfilled = wakeUp;
-      // Load the new wasm.
-      asm = createWasm();
+  emscripten_lazy_load_code__async: true,
+  emscripten_lazy_load_code: () => Asyncify.handleSleep((wakeUp) => {
+    // Update the expected wasm binary file to be the lazy one.
+    wasmBinaryFile += '.lazy.wasm';
+    // Add a callback for when all run dependencies are fulfilled, which happens when async wasm loading is done.
+    dependenciesFulfilled = wakeUp;
+    // Load the new wasm.
+    createWasm();
+  }),
+
+  _load_secondary_module__sig: 'v',
+  _load_secondary_module: async function() {
+    // Mark the module as loading for the wasm module (so it doesn't try to load it again).
+    wasmExports['load_secondary_module_status'].value = 1;
+    var imports = {'primary': wasmExports};
+    // Replace '.wasm' suffix with '.deferred.wasm'.
+    var deferred = wasmBinaryFile.slice(0, -5) + '.deferred.wasm';
+    await new Promise((resolve) => {
+      instantiateAsync(null, deferred, imports, resolve);
     });
   },
 
-  $Fibers__deps: ['$Asyncify'],
+  $Fibers__deps: ['$Asyncify', 'emscripten_stack_set_limits', '$stackRestore'],
   $Fibers: {
     nextFiber: 0,
     trampolineRunning: false,
-    trampoline: function() {
+    trampoline() {
       if (!Fibers.trampolineRunning && Fibers.nextFiber) {
         Fibers.trampolineRunning = true;
         do {
           var fiber = Fibers.nextFiber;
           Fibers.nextFiber = 0;
 #if ASYNCIFY_DEBUG >= 2
-          err("ASYNCIFY/FIBER: trampoline jump into fiber", fiber, new Error().stack);
+          dbg("ASYNCIFY/FIBER: trampoline jump into fiber", fiber, new Error().stack);
 #endif
           Fibers.finishContextSwitch(fiber);
         } while (Fibers.nextFiber);
@@ -433,77 +545,51 @@ mergeInto(LibraryManager.library, {
     /*
      * NOTE: This function is the asynchronous part of emscripten_fiber_swap.
      */
-    finishContextSwitch: function(newFiber) {
-      var stack_base = {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_base,  'i32') }}};
-      var stack_max =  {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_limit, 'i32') }}};
+    finishContextSwitch(newFiber) {
+      var stack_base = {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_base,  '*') }}};
+      var stack_max =  {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_limit, '*') }}};
       _emscripten_stack_set_limits(stack_base, stack_max);
 
 #if STACK_OVERFLOW_CHECK >= 2
-      Module['___set_stack_limits'](stack_base, stack_max);
+      ___set_stack_limits(stack_base, stack_max);
 #endif
 
-      stackRestore({{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_ptr,   'i32') }}});
+      stackRestore({{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_ptr,   '*') }}});
 
-      var entryPoint = {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.entry, 'i32') }}};
+      var entryPoint = {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.entry, '*') }}};
 
       if (entryPoint !== 0) {
 #if STACK_OVERFLOW_CHECK
         writeStackCookie();
 #endif
 #if ASYNCIFY_DEBUG
-        err('ASYNCIFY/FIBER: entering fiber', newFiber, 'for the first time');
+        dbg('ASYNCIFY/FIBER: entering fiber', newFiber, 'for the first time');
 #endif
         Asyncify.currData = null;
-        {{{ makeSetValue('newFiber', C_STRUCTS.emscripten_fiber_s.entry, 0, 'i32') }}};
+        {{{ makeSetValue('newFiber', C_STRUCTS.emscripten_fiber_s.entry, 0, '*') }}};
 
-        var userData = {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.user_data, 'i32') }}};
-        {{{ makeDynCall('vi', 'entryPoint') }}}(userData);
+        var userData = {{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.user_data, '*') }}};
+        {{{ makeDynCall('vp', 'entryPoint') }}}(userData);
       } else {
         var asyncifyData = newFiber + {{{ C_STRUCTS.emscripten_fiber_s.asyncify_data }}};
         Asyncify.currData = asyncifyData;
 
 #if ASYNCIFY_DEBUG
-        err('ASYNCIFY/FIBER: start rewind', asyncifyData, '(resuming fiber', newFiber, ')');
+        dbg('ASYNCIFY/FIBER: start rewind', asyncifyData, '(resuming fiber', newFiber, ')');
 #endif
         Asyncify.state = Asyncify.State.Rewinding;
-        Module['_asyncify_start_rewind'](asyncifyData);
+        _asyncify_start_rewind(asyncifyData);
         Asyncify.doRewind(asyncifyData);
       }
     },
   },
 
-  emscripten_fiber_init__sig: 'viiiiiii',
-  emscripten_fiber_init__deps: ['$Asyncify'],
-  emscripten_fiber_init: function(fiber, entryPoint, userData, cStack, cStackSize, asyncStack, asyncStackSize) {
-    var cStackBase = cStack + cStackSize;
-
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.stack_base,  'cStackBase',  'i32') }}};
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.stack_limit, 'cStack',      'i32') }}};
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.stack_ptr,   'cStackBase',  'i32') }}};
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.entry,       'entryPoint', 'i32') }}};
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.user_data,   'userData',   'i32') }}};
-
-    var asyncifyData = fiber + {{{ C_STRUCTS.emscripten_fiber_s.asyncify_data }}};
-    Asyncify.setDataHeader(asyncifyData, asyncStack, asyncStackSize);
-  },
-
-  emscripten_fiber_init_from_current_context__sig: 'vii',
-  emscripten_fiber_init_from_current_context__deps: ['$Asyncify'],
-  emscripten_fiber_init_from_current_context: function(fiber, asyncStack, asyncStackSize) {
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.stack_base,  '_emscripten_stack_get_base()', 'i32') }}};
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.stack_limit, '_emscripten_stack_get_end()',  'i32') }}};
-    {{{ makeSetValue('fiber', C_STRUCTS.emscripten_fiber_s.entry,       0,            'i32') }}};
-
-    var asyncifyData = fiber + {{{ C_STRUCTS.emscripten_fiber_s.asyncify_data }}};
-    Asyncify.setDataHeader(asyncifyData, asyncStack, asyncStackSize);
-  },
-
-  emscripten_fiber_swap__sig: 'vii',
-  emscripten_fiber_swap__deps: ["$Asyncify", "$Fibers"],
-  emscripten_fiber_swap: function(oldFiber, newFiber) {
+  emscripten_fiber_swap__deps: ["$Asyncify", "$Fibers", '$stackSave'],
+  emscripten_fiber_swap__async: true,
+  emscripten_fiber_swap: (oldFiber, newFiber) => {
     if (ABORT) return;
 #if ASYNCIFY_DEBUG
-    err('ASYNCIFY/FIBER: swap', oldFiber, '->', newFiber, 'state:', Asyncify.state);
+    dbg('ASYNCIFY/FIBER: swap', oldFiber, '->', newFiber, 'state:', Asyncify.state);
 #endif
     if (Asyncify.state === Asyncify.State.Normal) {
       Asyncify.state = Asyncify.State.Unwinding;
@@ -513,12 +599,12 @@ mergeInto(LibraryManager.library, {
       Asyncify.currData = asyncifyData;
 
 #if ASYNCIFY_DEBUG
-      err('ASYNCIFY/FIBER: start unwind', asyncifyData);
+      dbg('ASYNCIFY/FIBER: start unwind', asyncifyData);
 #endif
-      Module['_asyncify_start_unwind'](asyncifyData);
+      _asyncify_start_unwind(asyncifyData);
 
       var stackTop = stackSave();
-      {{{ makeSetValue('oldFiber', C_STRUCTS.emscripten_fiber_s.stack_ptr, 'stackTop', 'i32') }}};
+      {{{ makeSetValue('oldFiber', C_STRUCTS.emscripten_fiber_s.stack_ptr, 'stackTop', '*') }}};
 
       Fibers.nextFiber = newFiber;
     } else {
@@ -526,38 +612,32 @@ mergeInto(LibraryManager.library, {
       assert(Asyncify.state === Asyncify.State.Rewinding);
 #endif
 #if ASYNCIFY_DEBUG
-      err('ASYNCIFY/FIBER: stop rewind');
+      dbg('ASYNCIFY/FIBER: stop rewind');
 #endif
       Asyncify.state = Asyncify.State.Normal;
-      Module['_asyncify_stop_rewind']();
+      _asyncify_stop_rewind();
       Asyncify.currData = null;
     }
   },
 #else // ASYNCIFY
-  emscripten_sleep: function() {
+  emscripten_sleep: () => {
     throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_sleep';
   },
-  emscripten_wget: function() {
+  emscripten_wget: (url, file) => {
     throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_wget';
   },
-  emscripten_wget_data: function() {
+  emscripten_wget_data: (url, pbuffer, pnum, perror) => {
     throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_wget_data';
   },
-  emscripten_scan_registers: function() {
+  emscripten_scan_registers: (func) => {
     throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_scan_registers';
   },
-  emscripten_fiber_init: function() {
-    throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_fiber_init';
-  },
-  emscripten_fiber_init_from_current_context: function() {
-    throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_fiber_init_from_current_context';
-  },
-  emscripten_fiber_swap: function() {
+  emscripten_fiber_swap: function(oldFiber, newFiber) {
     throw 'Please compile your program with async support in order to use asynchronous operations like emscripten_fiber_swap';
   },
 #endif // ASYNCIFY
 });
 
 if (ASYNCIFY) {
-  DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.push('$Asyncify');
+  extraLibraryFuncs.push('$Asyncify');
 }
