@@ -15,6 +15,15 @@
  * value of 62006. (multiple creations silently return the same surface)
  */
 
+{{{
+// Magic ID for Emscripten 'default display' 
+globalThis.eglDefaultDisplay = 62000;
+// Magic ID for the only EGLConfig supported by Emscripten
+globalThis.eglDefaultConfig = 62002;
+// Magic ID for Emscripten EGLContext
+globalThis.eglDefaultContext = 62004;
+}}}
+
 var LibraryEGL = {
   $EGL__deps: ['$Browser'],
   $EGL: {
@@ -34,12 +43,12 @@ var LibraryEGL = {
 
     stringCache: {},
 
-    setErrorCode: function(code) {
+    setErrorCode(code) {
       EGL.errorCode = code;
     },
 
-    chooseConfig: function(display, attribList, config, config_size, numConfigs) {
-      if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+    chooseConfig(display, attribList, config, config_size, numConfigs) {
+      if (display != {{{ eglDefaultDisplay }}}) {
         EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
         return 0;
       }
@@ -81,7 +90,7 @@ var LibraryEGL = {
         {{{ makeSetValue('numConfigs', '0', '1', 'i32') }}}; // Total number of supported configs: 1.
       }
       if (config && config_size > 0) {
-        {{{ makeSetValue('config', '0', '62002' /* Magic ID for the only EGLConfig supported by Emscripten */, 'i32') }}};
+        {{{ makeSetValue('config', '0', eglDefaultConfig /* Magic ID for the only EGLConfig supported by Emscripten */, '*') }}};
       }
 
       EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
@@ -91,49 +100,39 @@ var LibraryEGL = {
 
   // EGLAPI EGLDisplay EGLAPIENTRY eglGetDisplay(EGLNativeDisplayType display_id);
   eglGetDisplay__proxy: 'sync',
-  eglGetDisplay__sig: 'ii',
-  eglGetDisplay: function(nativeDisplayType) {
+  eglGetDisplay: (nativeDisplayType) => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
-    // Note: As a 'conformant' implementation of EGL, we would prefer to init here only if the user
-    //       calls this function with EGL_DEFAULT_DISPLAY. Other display IDs would be preferred to be unsupported
-    //       and EGL_NO_DISPLAY returned. Uncomment the following code lines to do this.
-    // Instead, an alternative route has been preferred, namely that the Emscripten EGL implementation
-    // "emulates" X11, and eglGetDisplay is expected to accept/receive a pointer to an X11 Display object.
-    // Therefore, be lax and allow anything to be passed in, and return the magic handle to our default EGLDisplay object.
-
-//    if (nativeDisplayType == 0 /* EGL_DEFAULT_DISPLAY */) {
-        return 62000; // Magic ID for Emscripten 'default display'
-//    }
-//    else
-//      return 0; // EGL_NO_DISPLAY
+    // Emscripten EGL implementation "emulates" X11, and eglGetDisplay is
+    // expected to accept/receive a pointer to an X11 Display object (or
+    // EGL_DEFAULT_DISPLAY).
+    if (nativeDisplayType != 0 /* EGL_DEFAULT_DISPLAY */ && nativeDisplayType != 1 /* see library_xlib.js */) {
+      return 0; // EGL_NO_DISPLAY
+    }
+    return {{{ eglDefaultDisplay }}};
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglInitialize(EGLDisplay dpy, EGLint *major, EGLint *minor);
   eglInitialize__proxy: 'sync',
-  eglInitialize__sig: 'iiii',
-  eglInitialize: function(display, majorVersion, minorVersion) {
-    if (display == 62000 /* Magic ID for Emscripten 'default display' */) {
-      if (majorVersion) {
-        {{{ makeSetValue('majorVersion', '0', '1', 'i32') }}}; // Advertise EGL Major version: '1'
-      }
-      if (minorVersion) {
-        {{{ makeSetValue('minorVersion', '0', '4', 'i32') }}}; // Advertise EGL Minor version: '4'
-      }
-      EGL.defaultDisplayInitialized = true;
-      EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
-      return 1;
-    }
-    else {
+  eglInitialize: (display, majorVersion, minorVersion) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
+    if (majorVersion) {
+      {{{ makeSetValue('majorVersion', '0', '1', 'i32') }}}; // Advertise EGL Major version: '1'
+    }
+    if (minorVersion) {
+      {{{ makeSetValue('minorVersion', '0', '4', 'i32') }}}; // Advertise EGL Minor version: '4'
+    }
+    EGL.defaultDisplayInitialized = true;
+    EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
+    return 1;
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglTerminate(EGLDisplay dpy);
   eglTerminate__proxy: 'sync',
-  eglTerminate__sig: 'ii',
-  eglTerminate: function(display) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglTerminate: (display) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
@@ -147,27 +146,24 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglGetConfigs(EGLDisplay dpy, EGLConfig *configs, EGLint config_size, EGLint *num_config);
   eglGetConfigs__proxy: 'sync',
-  eglGetConfigs__sig: 'iiiii',
-  eglGetConfigs: function(display, configs, config_size, numConfigs) {
+  eglGetConfigs: (display, configs, config_size, numConfigs) => {
     return EGL.chooseConfig(display, 0, configs, config_size, numConfigs);
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglChooseConfig(EGLDisplay dpy, const EGLint *attrib_list, EGLConfig *configs, EGLint config_size, EGLint *num_config);
   eglChooseConfig__proxy: 'sync',
-  eglChooseConfig__sig: 'iiiiii',
-  eglChooseConfig: function(display, attrib_list, configs, config_size, numConfigs) {
+  eglChooseConfig: (display, attrib_list, configs, config_size, numConfigs) => {
     return EGL.chooseConfig(display, attrib_list, configs, config_size, numConfigs);
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglGetConfigAttrib(EGLDisplay dpy, EGLConfig config, EGLint attribute, EGLint *value);
   eglGetConfigAttrib__proxy: 'sync',
-  eglGetConfigAttrib__sig: 'iiiii',
-  eglGetConfigAttrib: function(display, config, attribute, value) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglGetConfigAttrib: (display, config, attribute, value) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
-    if (config != 62002 /* Magic ID for the only EGLConfig supported by Emscripten */) {
+    if (config != {{{ eglDefaultConfig }}}) {
       EGL.setErrorCode(0x3005 /* EGL_BAD_CONFIG */);
       return 0;
     }
@@ -203,7 +199,7 @@ var LibraryEGL = {
       {{{ makeSetValue('value', '0', '0x3038' /* EGL_NONE */, 'i32') }}};
       return 1;
     case 0x3028: // EGL_CONFIG_ID
-      {{{ makeSetValue('value', '0', '62002' /* Magic ID for the only EGLConfig supported by Emscripten */, 'i32') }}};
+      {{{ makeSetValue('value', '0', eglDefaultConfig, 'i32') }}};
       return 1;
     case 0x3029: // EGL_LEVEL
       {{{ makeSetValue('value', '0', '0' /* Z order/depth layer for this level. Not applicable for Emscripten. */, 'i32') }}};
@@ -279,13 +275,12 @@ var LibraryEGL = {
 
   // EGLAPI EGLSurface EGLAPIENTRY eglCreateWindowSurface(EGLDisplay dpy, EGLConfig config, EGLNativeWindowType win, const EGLint *attrib_list);
   eglCreateWindowSurface__proxy: 'sync',
-  eglCreateWindowSurface__sig: 'iiiii',
-  eglCreateWindowSurface: function(display, config, win, attrib_list) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglCreateWindowSurface: (display, config, win, attrib_list) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
-    if (config != 62002 /* Magic ID for the only EGLConfig supported by Emscripten */) {
+    if (config != {{{ eglDefaultConfig }}}) {
       EGL.setErrorCode(0x3005 /* EGL_BAD_CONFIG */);
       return 0;
     }
@@ -299,9 +294,8 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglDestroySurface(EGLDisplay display, EGLSurface surface);
   eglDestroySurface__proxy: 'sync',
-  eglDestroySurface__sig: 'iii',
-  eglDestroySurface: function(display, surface) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglDestroySurface: (display, surface) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
@@ -323,9 +317,8 @@ var LibraryEGL = {
 
   // EGLAPI EGLContext EGLAPIENTRY eglCreateContext(EGLDisplay dpy, EGLConfig config, EGLContext share_context, const EGLint *attrib_list);
   eglCreateContext__proxy: 'sync',
-  eglCreateContext__sig: 'iiiii',
-  eglCreateContext: function(display, config, hmm, contextAttribs) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglCreateContext: (display, config, hmm, contextAttribs) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
@@ -353,9 +346,9 @@ var LibraryEGL = {
 #endif
 #if GL_ASSERTIONS
       if (glesContextVersion == 3) {
-        err('When initializing GLES3/WebGL2 via EGL, one must build with -s MAX_WEBGL_VERSION=2 !');
+        err('When initializing GLES3/WebGL2 via EGL, one must build with -sMAX_WEBGL_VERSION=2!');
       } else {
-        err('When initializing GLES2/WebGL1 via EGL, one must pass EGL_CONTEXT_CLIENT_VERSION = 2 to GL context attributes! GLES version ' + glesContextVersion + ' is not supported!');
+        err(`When initializing GLES2/WebGL1 via EGL, one must pass EGL_CONTEXT_CLIENT_VERSION = 2 to GL context attributes! GLES version ${glesContextVersion} is not supported!`);
       }
 #endif
       EGL.setErrorCode(0x3005 /* EGL_BAD_CONFIG */);
@@ -372,12 +365,12 @@ var LibraryEGL = {
 
       // Run callbacks so that GL emulation works
       GL.makeContextCurrent(EGL.context);
-      Module.useWebGL = true;
-      Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
+      Browser.useWebGL = true;
+      Browser.moduleContextCreatedCallbacks.forEach((callback) => callback());
 
       // Note: This function only creates a context, but it shall not make it active.
       GL.makeContextCurrent(null);
-      return 62004; // Magic ID for Emscripten EGLContext
+      return {{{ eglDefaultContext }}};
     } else {
       EGL.setErrorCode(0x3009 /* EGL_BAD_MATCH */); // By the EGL 1.4 spec, an implementation that does not support GLES2 (WebGL in this case), this error code is set.
       return 0; /* EGL_NO_CONTEXT */
@@ -388,13 +381,12 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglDestroyContext(EGLDisplay dpy, EGLContext context);
   eglDestroyContext__proxy: 'sync',
-  eglDestroyContext__sig: 'iii',
-  eglDestroyContext: function(display, context) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglDestroyContext: (display, context) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
-    if (context != 62004 /* Magic ID for Emscripten EGLContext */) {
+    if (context != {{{ eglDefaultContext }}}) {
       EGL.setErrorCode(0x3006 /* EGL_BAD_CONTEXT */);
       return 0;
     }
@@ -409,9 +401,8 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglQuerySurface(EGLDisplay dpy, EGLSurface surface, EGLint attribute, EGLint *value);
   eglQuerySurface__proxy: 'sync',
-  eglQuerySurface__sig: 'iiiii',
-  eglQuerySurface: function(display, surface, attribute, value) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglQuerySurface: (display, surface, attribute, value) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
@@ -426,7 +417,7 @@ var LibraryEGL = {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     switch (attribute) {
     case 0x3028: // EGL_CONFIG_ID
-      {{{ makeSetValue('value', '0', '62002' /* A magic value for the only EGLConfig configuration ID supported by Emscripten. */, 'i32') }}};
+      {{{ makeSetValue('value', '0', eglDefaultConfig, 'i32') }}};
         return 1;
     case 0x3058: // EGL_LARGEST_PBUFFER
       // Odd EGL API: If surface is not a pbuffer surface, 'value' should not be written to. It's not specified as an error, so true should(?) be returned.
@@ -476,14 +467,13 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglQueryContext(EGLDisplay dpy, EGLContext ctx, EGLint attribute, EGLint *value);
   eglQueryContext__proxy: 'sync',
-  eglQueryContext__sig: 'iiiii',
-  eglQueryContext: function(display, context, attribute, value) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglQueryContext: (display, context, attribute, value) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
     //\todo An EGL_NOT_INITIALIZED error is generated if EGL is not initialized for dpy.
-    if (context != 62004 /* Magic ID for Emscripten EGLContext */) {
+    if (context != {{{ eglDefaultContext }}}) {
       EGL.setErrorCode(0x3006 /* EGL_BAD_CONTEXT */);
       return 0;
     }
@@ -495,7 +485,7 @@ var LibraryEGL = {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     switch (attribute) {
       case 0x3028: // EGL_CONFIG_ID
-        {{{ makeSetValue('value', '0', '62002' /* A magic value for the only EGLConfig configuration ID supported by Emscripten. */, 'i32') }}};
+        {{{ makeSetValue('value', '0', eglDefaultConfig, 'i32') }}};
         return 1;
       case 0x3097: // EGL_CONTEXT_CLIENT_TYPE
         {{{ makeSetValue('value', '0', '0x30A0' /* EGL_OPENGL_ES_API */, 'i32') }}};
@@ -516,19 +506,13 @@ var LibraryEGL = {
 
   // EGLAPI EGLint EGLAPIENTRY eglGetError(void);
   eglGetError__proxy: 'sync',
-  eglGetError__sig: 'i',
-  eglGetError: function() {
-    return EGL.errorCode;
-  },
+  eglGetError: () => EGL.errorCode,
 
   // EGLAPI const char * EGLAPIENTRY eglQueryString(EGLDisplay dpy, EGLint name);
-#if MINIMAL_RUNTIME
-  eglQueryString__deps: ['$allocateUTF8'],
-#endif
+  eglQueryString__deps: ['$stringToNewUTF8'],
   eglQueryString__proxy: 'sync',
-  eglQueryString__sig: 'iii',
-  eglQueryString: function(display, name) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglQueryString: (display, name) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
@@ -537,10 +521,10 @@ var LibraryEGL = {
     if (EGL.stringCache[name]) return EGL.stringCache[name];
     var ret;
     switch (name) {
-      case 0x3053 /* EGL_VENDOR */: ret = allocateUTF8("Emscripten"); break;
-      case 0x3054 /* EGL_VERSION */: ret = allocateUTF8("1.4 Emscripten EGL"); break;
-      case 0x3055 /* EGL_EXTENSIONS */:  ret = allocateUTF8(""); break; // Currently not supporting any EGL extensions.
-      case 0x308D /* EGL_CLIENT_APIS */: ret = allocateUTF8("OpenGL_ES"); break;
+      case 0x3053 /* EGL_VENDOR */: ret = stringToNewUTF8("Emscripten"); break;
+      case 0x3054 /* EGL_VERSION */: ret = stringToNewUTF8("1.4 Emscripten EGL"); break;
+      case 0x3055 /* EGL_EXTENSIONS */:  ret = stringToNewUTF8(""); break; // Currently not supporting any EGL extensions.
+      case 0x308D /* EGL_CLIENT_APIS */: ret = stringToNewUTF8("OpenGL_ES"); break;
       default:
         EGL.setErrorCode(0x300C /* EGL_BAD_PARAMETER */);
         return 0;
@@ -551,37 +535,33 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglBindAPI(EGLenum api);
   eglBindAPI__proxy: 'sync',
-  eglBindAPI__sig: 'ii',
-  eglBindAPI: function(api) {
+  eglBindAPI: (api) => {
     if (api == 0x30A0 /* EGL_OPENGL_ES_API */) {
       EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
       return 1;
-    } else { // if (api == 0x30A1 /* EGL_OPENVG_API */ || api == 0x30A2 /* EGL_OPENGL_API */) {
-      EGL.setErrorCode(0x300C /* EGL_BAD_PARAMETER */);
-      return 0;
     }
+    // if (api == 0x30A1 /* EGL_OPENVG_API */ || api == 0x30A2 /* EGL_OPENGL_API */) {
+    EGL.setErrorCode(0x300C /* EGL_BAD_PARAMETER */);
+    return 0;
   },
 
   // EGLAPI EGLenum EGLAPIENTRY eglQueryAPI(void);
   eglQueryAPI__proxy: 'sync',
-  eglQueryAPI__sig: 'i',
-  eglQueryAPI: function() {
+  eglQueryAPI: () => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 0x30A0; // EGL_OPENGL_ES_API
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglWaitClient(void);
   eglWaitClient__proxy: 'sync',
-  eglWaitClient__sig: 'i',
-  eglWaitClient: function() {
+  eglWaitClient: () => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 1;
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglWaitNative(EGLint engine);
   eglWaitNative__proxy: 'sync',
-  eglWaitNative__sig: 'ii',
-  eglWaitNative: function(nativeEngineId) {
+  eglWaitNative: (nativeEngineId) => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 1;
   },
@@ -593,14 +573,13 @@ var LibraryEGL = {
   // EGLAPI EGLBoolean EGLAPIENTRY eglSwapInterval(EGLDisplay dpy, EGLint interval);
   eglSwapInterval__deps: ['emscripten_set_main_loop_timing'],
   eglSwapInterval__proxy: 'sync',
-  eglSwapInterval__sig: 'iii',
-  eglSwapInterval: function(display, interval) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglSwapInterval: (display, interval) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
-    if (interval == 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 0);
-    else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, interval);
+    if (interval == 0) _emscripten_set_main_loop_timing({{{ cDefs.EM_TIMING_SETTIMEOUT }}}, 0);
+    else _emscripten_set_main_loop_timing({{{ cDefs.EM_TIMING_RAF }}}, interval);
 
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 1;
@@ -609,14 +588,13 @@ var LibraryEGL = {
   // EGLAPI EGLBoolean EGLAPIENTRY eglMakeCurrent(EGLDisplay dpy, EGLSurface draw, EGLSurface read, EGLContext ctx);
   eglMakeCurrent__deps: ['$GL'],
   eglMakeCurrent__proxy: 'sync',
-  eglMakeCurrent__sig: 'iiiii',
-  eglMakeCurrent: function(display, draw, read, context) {
-    if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
+  eglMakeCurrent: (display, draw, read, context) => {
+    if (display != {{{ eglDefaultDisplay }}}) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0 /* EGL_FALSE */;
     }
     //\todo An EGL_NOT_INITIALIZED error is generated if EGL is not initialized for dpy.
-    if (context != 0 && context != 62004 /* Magic ID for Emscripten EGLContext */) {
+    if (context != 0 && context != {{{ eglDefaultContext }}}) {
       EGL.setErrorCode(0x3006 /* EGL_BAD_CONTEXT */);
       return 0;
     }
@@ -636,15 +614,11 @@ var LibraryEGL = {
 
   // EGLAPI EGLContext EGLAPIENTRY eglGetCurrentContext(void);
   eglGetCurrentContext__proxy: 'sync',
-  eglGetCurrentContext__sig: 'i',
-  eglGetCurrentContext: function() {
-    return EGL.currentContext;
-  },
+  eglGetCurrentContext: () => EGL.currentContext,
 
   // EGLAPI EGLSurface EGLAPIENTRY eglGetCurrentSurface(EGLint readdraw);
   eglGetCurrentSurface__proxy: 'sync',
-  eglGetCurrentSurface__sig: 'ii',
-  eglGetCurrentSurface: function(readdraw) {
+  eglGetCurrentSurface: (readdraw) => {
     if (readdraw == 0x305A /* EGL_READ */) {
       return EGL.currentReadSurface;
     } else if (readdraw == 0x3059 /* EGL_DRAW */) {
@@ -657,15 +631,11 @@ var LibraryEGL = {
 
   // EGLAPI EGLDisplay EGLAPIENTRY eglGetCurrentDisplay(void);
   eglGetCurrentDisplay__proxy: 'sync',
-  eglGetCurrentDisplay__sig: 'i',
-  eglGetCurrentDisplay: function() {
-    return EGL.currentContext ? 62000 /* Magic ID for Emscripten 'default display' */ : 0;
-  },
+  eglGetCurrentDisplay: () => EGL.currentContext ? {{{ eglDefaultDisplay }}} : 0,
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglSwapBuffers(EGLDisplay dpy, EGLSurface surface);
   eglSwapBuffers__proxy: 'sync',
-  eglSwapBuffers__sig: 'iii',
-  eglSwapBuffers: function() {
+  eglSwapBuffers: (dpy, surface) => {
 #if PROXY_TO_WORKER
     if (Browser.doSwapBuffers) Browser.doSwapBuffers();
 #endif
@@ -688,8 +658,7 @@ var LibraryEGL = {
   },
 
   eglReleaseThread__proxy: 'sync',
-  eglReleaseThread__sig: 'i',
-  eglReleaseThread: function() {
+  eglReleaseThread: () => {
     // Equivalent to eglMakeCurrent with EGL_NO_CONTEXT and EGL_NO_SURFACE.
     EGL.currentContext = 0;
     EGL.currentReadSurface = 0;
@@ -704,4 +673,4 @@ var LibraryEGL = {
 
 autoAddDeps(LibraryEGL, '$EGL');
 
-mergeInto(LibraryManager.library, LibraryEGL);
+addToLibrary(LibraryEGL);
