@@ -35,7 +35,7 @@ var LibraryPThread = {
   $PThread__postset: 'PThread.init();',
   $PThread__deps: ['_emscripten_thread_init',
                    '$terminateWorker',
-                   '$cleanupThread', '$zeroMemory',
+                   '$cleanupThread',
 #if MAIN_MODULE
                    '$markAsFinished',
 #endif
@@ -75,7 +75,7 @@ var LibraryPThread = {
         ) {
           t = _pthread_self();
         }
-        return 'w:' + workerID + ',t:' + ptrToString(t) + ': ';
+        return `w:${workerID},t:${ptrToString(t)}: `;
       }
 
       // Prefix all err()/dbg() messages with the calling thread ID.
@@ -94,8 +94,6 @@ var LibraryPThread = {
 #endif
       if ({{{ ENVIRONMENT_IS_MAIN_THREAD() }}}) {
         PThread.initMainThread();
-      } else {
-        PThread.initWorker();
       }
     },
     initMainThread() {
@@ -122,19 +120,6 @@ var LibraryPThread = {
 #endif
     },
 
-    initWorker() {
-#if isSymbolNeeded('$noExitRuntime')
-      // The default behaviour for pthreads is always to exit once they return
-      // from their entry point (or call pthread_exit).  If we set noExitRuntime
-      // to true here on pthreads they would never complete and attempt to
-      // pthread_join to them would block forever.
-      // pthreads can still choose to set `noExitRuntime` explicitly, or
-      // call emscripten_unwind_to_js_event_loop to extend their lifetime beyond
-      // their main function.  See comment in src/runtime_pthread.js for more.
-      noExitRuntime = false;
-#endif
-    },
-
 #if PTHREADS_PROFILING
     getThreadName(pthreadPtr) {
       var profilerBlock = {{{ makeGetValue('pthreadPtr', C_STRUCTS.pthread.profilerBlock, '*') }}};
@@ -244,16 +229,16 @@ var LibraryPThread = {
     loadWasmModuleToWorker: (worker) => new Promise((onFinishedLoading) => {
       worker.onmessage = (e) => {
         var d = e['data'];
-        var cmd = d['cmd'];
+        var cmd = d.cmd;
 
         // If this message is intended to a recipient that is not the main
         // thread, forward it to the target thread.
-        if (d['targetThread'] && d['targetThread'] != _pthread_self()) {
-          var targetWorker = PThread.pthreads[d['targetThread']];
+        if (d.targetThread && d.targetThread != _pthread_self()) {
+          var targetWorker = PThread.pthreads[d.targetThread];
           if (targetWorker) {
-            targetWorker.postMessage(d, d['transferList']);
+            targetWorker.postMessage(d, d.transferList);
           } else {
-            err(`Internal error! Worker sent a message "${cmd}" to target pthread ${d['targetThread']}, but that thread no longer exists!`);
+            err(`Internal error! Worker sent a message "${cmd}" to target pthread ${d.targetThread}, but that thread no longer exists!`);
           }
           return;
         }
@@ -263,10 +248,10 @@ var LibraryPThread = {
         } else if (cmd === 'spawnThread') {
           spawnThread(d);
         } else if (cmd === 'cleanupThread') {
-          cleanupThread(d['thread']);
+          cleanupThread(d.thread);
 #if MAIN_MODULE
         } else if (cmd === 'markAsFinished') {
-          markAsFinished(d['thread']);
+          markAsFinished(d.thread);
 #endif
         } else if (cmd === 'loaded') {
           worker.loaded = true;
@@ -281,13 +266,13 @@ var LibraryPThread = {
 #endif
           onFinishedLoading(worker);
         } else if (cmd === 'alert') {
-          alert(`Thread ${d['threadId']}: ${d['text']}`);
+          alert(`Thread ${d.threadId}: ${d.text}`);
         } else if (d.target === 'setimmediate') {
           // Worker wants to postMessage() to itself to implement setImmediate()
           // emulation.
           worker.postMessage(d);
         } else if (cmd === 'callHandler') {
-          Module[d['handler']](...d['args']);
+          Module[d.handler](...d.args);
         } else if (cmd) {
           // The received message looks like something that should be handled by this message
           // handler, (since there is a e.data.cmd field present), but is not one of the
@@ -348,28 +333,28 @@ var LibraryPThread = {
 
       // Ask the new worker to load up the Emscripten-compiled page. This is a heavy operation.
       worker.postMessage({
-        'cmd': 'load',
-        'handlers': handlers,
+        cmd: 'load',
+        handlers: handlers,
 #if WASM2JS
         // the polyfill WebAssembly.Memory instance has function properties,
         // which will fail in postMessage, so just send a custom object with the
         // property we need, the buffer
-        'wasmMemory': { 'buffer': wasmMemory.buffer },
+        wasmMemory: { 'buffer': wasmMemory.buffer },
 #else // WASM2JS
-        'wasmMemory': wasmMemory,
+        wasmMemory,
 #endif // WASM2JS
-        'wasmModule': wasmModule,
+        wasmModule,
 #if LOAD_SOURCE_MAP
-        'wasmSourceMap': wasmSourceMap,
+        wasmSourceMap,
 #endif
 #if USE_OFFSET_CONVERTER
-        'wasmOffsetConverter': wasmOffsetConverter,
+        wasmOffsetConverter,
 #endif
 #if MAIN_MODULE
-        'dynamicLibraries': dynamicLibraries,
+        dynamicLibraries,
         // Share all modules that have been loaded so far.  New workers
         // won't start running threads until these are all loaded.
-        'sharedModules': sharedModules,
+        sharedModules,
 #endif
 #if ASSERTIONS
         'workerID': worker.workerID,
@@ -420,13 +405,17 @@ var LibraryPThread = {
 #if ENVIRONMENT_MAY_BE_WEB || ENVIRONMENT_MAY_BE_WORKER
         // This is the way that we signal to the Web Worker that it is hosting
         // a pthread.
+#if ASSERTIONS
+        'name': 'em-pthread-' + PThread.nextWorkerID,
+#else
         'name': 'em-pthread',
+#endif
 #endif
       };
 #if EXPORT_ES6 && USE_ES6_IMPORT_META
       // If we're using module output, use bundler-friendly pattern.
 #if PTHREADS_DEBUG
-      dbg('Allocating a new web worker from ' + import.meta.url);
+      dbg(`Allocating a new web worker from ${import.meta.url}`);
 #endif
 #if TRUSTED_TYPES
       // Use Trusted Types compatible wrappers.
@@ -518,7 +507,7 @@ var LibraryPThread = {
     // the onmessage handlers if the message was coming from valid worker.
     worker.onmessage = (e) => {
 #if ASSERTIONS
-      var cmd = e['data']['cmd'];
+      var cmd = e['data'].cmd;
       err(`received "${cmd}" command from terminated worker: ${worker.workerID}`);
 #endif
     };
@@ -534,7 +523,7 @@ var LibraryPThread = {
     dbg(`_emscripten_thread_cleanup: ${ptrToString(thread)}`)
 #endif
     if (!ENVIRONMENT_IS_PTHREAD) cleanupThread(thread);
-    else postMessage({ 'cmd': 'cleanupThread', 'thread': thread });
+    else postMessage({ cmd: 'cleanupThread', thread });
   },
 
   _emscripten_thread_set_strongref: (thread) => {
@@ -633,10 +622,10 @@ var LibraryPThread = {
 
     worker.pthread_ptr = threadParams.pthread_ptr;
     var msg = {
-        'cmd': 'run',
-        'start_routine': threadParams.startRoutine,
-        'arg': threadParams.arg,
-        'pthread_ptr': threadParams.pthread_ptr,
+        cmd: 'run',
+        start_routine: threadParams.startRoutine,
+        arg: threadParams.arg,
+        pthread_ptr: threadParams.pthread_ptr,
     };
 #if OFFSCREENCANVAS_SUPPORT
     // Note that we do not need to quote these names because they are only used
@@ -701,13 +690,16 @@ var LibraryPThread = {
   __pthread_create_js__noleakcheck: true,
 #endif
   __pthread_create_js__deps: ['$spawnThread', 'pthread_self', '$pthreadCreateProxied',
+    'emscripten_has_threading_support',
 #if OFFSCREENCANVAS_SUPPORT
     'malloc',
 #endif
   ],
   __pthread_create_js: (pthread_ptr, attr, startRoutine, arg) => {
-    if (typeof SharedArrayBuffer == 'undefined') {
-      err('Current environment does not support SharedArrayBuffer, pthreads are not available!');
+    if (!_emscripten_has_threading_support()) {
+#if ASSERTIONS
+      dbg('pthread_create: environment does not support SharedArrayBuffer, pthreads are not available');
+#endif
       return {{{ cDefs.EAGAIN }}};
     }
 #if PTHREADS_DEBUG
@@ -731,17 +723,19 @@ var LibraryPThread = {
       transferredCanvasNames = '{{{ OFFSCREENCANVASES_TO_PTHREAD }}}';
     } else
 #endif
-    transferredCanvasNames &&= UTF8ToString(transferredCanvasNames).trim();
-    transferredCanvasNames &&= transferredCanvasNames.split(',');
+    {
+      transferredCanvasNames = UTF8ToString(transferredCanvasNames).trim();
+    }
+    transferredCanvasNames = transferredCanvasNames ? transferredCanvasNames.split(',') : [];
 #if GL_DEBUG
     dbg(`pthread_create: transferredCanvasNames="${transferredCanvasNames}"`);
 #endif
 
     var offscreenCanvases = {}; // Dictionary of OffscreenCanvas objects we'll transfer to the created thread to own
-    var moduleCanvasId = Module['canvas'] ? Module['canvas'].id : '';
+    var moduleCanvasId = Module['canvas']?.id || '';
     // Note that transferredCanvasNames might be null (so we cannot do a for-of loop).
-    for (var i in transferredCanvasNames) {
-      var name = transferredCanvasNames[i].trim();
+    for (var name of transferredCanvasNames) {
+      name = name.trim();
       var offscreenCanvasInfo;
       try {
         if (name == '#canvas') {
@@ -773,7 +767,7 @@ var LibraryPThread = {
           }
           if (canvas.transferControlToOffscreen) {
 #if GL_DEBUG
-            dbg('pthread_create: canvas.transferControlToOffscreen(), transferring canvas by name "' + name + '" (DOM id="' + canvas.id + '") from main thread to pthread');
+            dbg(`pthread_create: canvas.transferControlToOffscreen(), transferring canvas by name "${name}" (DOM id="${canvas.id}") from main thread to pthread`);
 #endif
             // Create a shared information block in heap so that we can control
             // the canvas size from any thread.
@@ -1066,6 +1060,16 @@ var LibraryPThread = {
     // `runtimeKeepaliveCounter` to zero. Reset it now so the new thread won't
     // be affected.
     runtimeKeepaliveCounter = 0;
+
+#if isSymbolNeeded('$noExitRuntime')
+    // Same for noExitRuntime.  The default for pthreads should always be false
+    // otherwise pthreads would never complete and attempts to pthread_join to
+    // them would block forever.
+    // pthreads can still choose to set `noExitRuntime` explicitly, or
+    // call emscripten_unwind_to_js_event_loop to extend their lifetime beyond
+    // their main function.  See comment in src/runtime_pthread.js for more.
+    noExitRuntime = 0;
+#endif
 #endif
 
 #if MAIN_MODULE
@@ -1115,7 +1119,7 @@ var LibraryPThread = {
     // running, but remain around waiting to be joined.  In this state they
     // cannot run any more proxied work.
     if (!ENVIRONMENT_IS_PTHREAD) markAsFinished(thread);
-    else postMessage({ 'cmd': 'markAsFinished', 'thread': thread });
+    else postMessage({ cmd: 'markAsFinished', thread });
   },
 
   $markAsFinished: (pthread_ptr) => {
@@ -1247,20 +1251,20 @@ var LibraryPThread = {
   // new thread that has not had a chance to initialize itself and execute
   // Atomics.waitAsync to prepare for the notification.
   _emscripten_notify_mailbox_postmessage__deps: ['$checkMailbox'],
-  _emscripten_notify_mailbox_postmessage: (targetThreadId, currThreadId, mainThreadId) => {
-    if (targetThreadId == currThreadId) {
+  _emscripten_notify_mailbox_postmessage: (targetThread, currThreadId) => {
+    if (targetThread == currThreadId) {
       setTimeout(checkMailbox);
     } else if (ENVIRONMENT_IS_PTHREAD) {
-      postMessage({'targetThread' : targetThreadId, 'cmd' : 'checkMailbox'});
+      postMessage({targetThread, cmd: 'checkMailbox'});
     } else {
-      var worker = PThread.pthreads[targetThreadId];
+      var worker = PThread.pthreads[targetThread];
       if (!worker) {
 #if ASSERTIONS
-        err(`Cannot send message to thread with ID ${targetThreadId}, unknown thread ID!`);
+        err(`Cannot send message to thread with ID ${targetThread}, unknown thread ID!`);
 #endif
         return;
       }
-      worker.postMessage({'cmd' : 'checkMailbox'});
+      worker.postMessage({cmd: 'checkMailbox'});
     }
   }
 };
