@@ -16,9 +16,25 @@
 #endif
 
 #if ENVIRONMENT_MAY_BE_NODE
-var ENVIRONMENT_IS_NODE = typeof process == 'object' && typeof process.versions == 'object' && typeof process.versions.node == 'string';
+var ENVIRONMENT_IS_NODE = typeof process == 'object' && typeof process.versions == 'object' && typeof process.versions.node == 'string' && process.type != 'renderer';
 if (ENVIRONMENT_IS_NODE) {
-  global.Worker = require('worker_threads').Worker;
+  var NodeWorker = require('worker_threads').Worker;
+  global.Worker = function(url, options) {
+    // Special handling for `data:` URL argument, to match the behaviour
+    // of the Web API.
+    if (typeof url == 'string' && url.startsWith('data:')) {
+#if EXPORT_ES6
+      // worker_threads always assume data URLs are ES6 modules
+      url = new URL(url);
+#else
+      // For class modules we decode the data URL and use `eval: true`.
+      url = Buffer.from(url.split(",")[1], 'base64').toString();
+      options ||= {}
+      options.eval = true;
+#endif
+    }
+    return new NodeWorker(url, options);
+  }
   var Module = Module || {}
 } else
 #endif
