@@ -19,7 +19,7 @@ WebAssembly = {
   // not a fully general polyfill.
   /** @constructor */
   Memory: function(opts) {
-#if USE_PTHREADS
+#if SHARED_MEMORY
     this.buffer = new SharedArrayBuffer(opts['initial'] * {{{ WASM_PAGE_SIZE }}});
 #else
     this.buffer = new ArrayBuffer(opts['initial'] * {{{ WASM_PAGE_SIZE }}});
@@ -63,10 +63,10 @@ WebAssembly = {
 
   /** @constructor */
   Instance: function(module, info) {
-    // TODO: use the module and info somehow - right now the wasm2js output is embedded in
+    // TODO: use the module somehow - right now the wasm2js output is embedded in
     // the main JS
     // This will be replaced by the actual wasm2js code.
-    this.exports = Module['__wasm2jsInstantiate__'](asmLibraryArg);
+    this.exports = Module['__wasm2jsInstantiate__'](info);
   },
 
   instantiate: /** @suppress{checkTypes} */ function(binary, info) {
@@ -74,10 +74,10 @@ WebAssembly = {
       then: function(ok) {
         var module = new WebAssembly.Module(binary);
         ok({
-#if USE_PTHREADS
+#if SHARED_MEMORY
           'module': module,
 #endif
-          'instance': new WebAssembly.Instance(module)
+          'instance': new WebAssembly.Instance(module, info)
         });
 #if ASSERTIONS || WASM == 2 // see postamble_minimal.js which uses .catch
         // Emulate a simple WebAssembly.instantiate(..).then(()=>{}).catch(()=>{}) syntax.
@@ -87,10 +87,9 @@ WebAssembly = {
     };
   },
 
-  RuntimeError: Error
-};
+  RuntimeError: Error,
 
 #if !MINIMAL_RUNTIME
-// We don't need to actually download a wasm binary, mark it as present but empty.
-wasmBinary = [];
+  isWasm2js: true,
 #endif
+};
