@@ -653,7 +653,7 @@ function WebGLWorker() {
   this.createShader = function(type) {
     var id = nextId++;
     commandBuffer.push(6, type, id);
-    return { id: id, what: 'shader', type: type };
+    return { id, what: 'shader', type };
   };
   this.deleteShader = function(shader) {
     if (!shader) return;
@@ -683,7 +683,7 @@ function WebGLWorker() {
     commandBuffer.push(12, program.id, shader.id);
   };
   this.bindAttribLocation = function(program, index, name) {
-    program.nextAttributes[name] = { what: 'attribute', name: name, size: -1, location: index, type: '?' }; // fill in size, type later
+    program.nextAttributes[name] = { what: 'attribute', name, size: -1, location: index, type: '?' }; // fill in size, type later
     program.nextAttributeVec[index] = name;
     commandBuffer.push(13, program.id, index, name);
   };
@@ -723,10 +723,10 @@ function WebGLWorker() {
       source = source.replace(/\n/g, '|\n'); // barrier between lines, to make regexing easier
       var newItems = source.match(new RegExp(type + '\\s+\\w+\\s+[\\w,\\s\[\\]]+;', 'g'));
       if (!newItems) return;
-      newItems.forEach(function(item) {
+      newItems.forEach((item) => {
         var m = new RegExp(type + '\\s+(\\w+)\\s+([\\w,\\s\[\\]]+);').exec(item);
         assert(m);
-        m[2].split(',').map(function(name) { name = name.trim(); return name.search(/\s/) >= 0 ? '' : name }).filter(function(name) { return !!name }).forEach(function(name) {
+        m[2].split(',').map((name) => { name = name.trim(); return name.search(/\s/) >= 0 ? '' : name }).filter((name) => !!name).forEach((name) => {
           var size = 1;
           var open = name.indexOf('[');
           var fullname = name;
@@ -737,8 +737,8 @@ function WebGLWorker() {
             fullname = name + '[0]';
           }
           if (!obj[name]) {
-            obj[name] = { what: type, name: fullname, size: size, location: -1, type: getTypeId(m[1]) };
-            if (vec) vec.push(name);
+            obj[name] = { what: type, name: fullname, size, location: -1, type: getTypeId(m[1]) };
+            vec?.push(name);
           }
         });
       });
@@ -754,7 +754,7 @@ function WebGLWorker() {
 
     var existingAttributes = {};
 
-    program.shaders.forEach(function(shader) {
+    program.shaders.forEach((shader) => {
       parseElementType(shader, 'uniform', program.uniforms, program.uniformVec);
       parseElementType(shader, 'attribute', existingAttributes, null);
     });
@@ -807,7 +807,7 @@ function WebGLWorker() {
     if (!(name in program.uniforms)) return null;
     var id = nextId++;
     commandBuffer.push(16, program.id, fullname, id);
-    return { what: 'location', uniform: program.uniforms[name], id: id, index: index };
+    return { what: 'location', uniform: program.uniforms[name], id, index };
   };
   this.getProgramInfoLog = function(shader) {
     return ''; // optimistic assumption of success; no proxying
@@ -1153,8 +1153,8 @@ function WebGLWorker() {
   Browser.doSwapBuffers = postRAF;
 
   var trueRAF = window.requestAnimationFrame;
-  window.requestAnimationFrame = function(func) {
-    trueRAF(function() {
+  window.requestAnimationFrame = (func) => {
+    trueRAF(() => {
       if (preRAF() === false) {
         window.requestAnimationFrame(func); // skip this frame, do it later
         return;
