@@ -10,6 +10,7 @@
 
 #include "file.h"
 #include <assert.h>
+#include <fcntl.h>
 #include <mutex>
 #include <utility>
 #include <vector>
@@ -29,21 +30,52 @@ template<typename T> bool addWillOverFlow(T a, T b) {
   return false;
 }
 
-// Access mode, file creation and file status flags for open.
-using oflags_t = uint32_t;
+class FileTable;
 
 class OpenFileState : public std::enable_shared_from_this<OpenFileState> {
   std::shared_ptr<File> file;
-  off_t position;
+  off_t position = 0;
   oflags_t flags; // RD_ONLY, WR_ONLY, RDWR
+
   // An OpenFileState needs a mutex if there are concurrent accesses on one open
   // file descriptor. This could occur if there are multiple seeks on the same
   // open file descriptor.
   std::recursive_mutex mutex;
 
+  // The number of times this OpenFileState appears in the table. Use this
+  // instead of shared_ptr::use_count to avoid accidentally counting temporary
+  // objects.
+  int uses = 0;
+
+  // We can't make the constructor private because std::make_shared needs to be
+  // able to call it, but we can make it unusable publicly.
+  struct private_key {
+    explicit private_key(int) {}
+  };
+
+  // `uses` is protected by the FileTable lock and can be accessed directly by
+  // `FileTable::Handle.
+  friend FileTable;
+
 public:
-  OpenFileState(size_t position, oflags_t flags, std::shared_ptr<File> file)
-    : position(position), flags(flags), file(file) {}
+  // Cache directory entries at the moment the directory is opened so that
+  // subsequent getdents calls have a stable view of the contents. Including
+  // files removed after the open and excluding files added after the open is
+  // allowed, and trying to recalculate the directory contents on each getdents
+  // call could lead to missed directory entries if there are concurrent
+  // deletions that effectively move entries back past the current read position
+  // in the open directory.
+  const std::vector<Directory::Entry> dirents;
+
+  OpenFileState(private_key,
+                oflags_t flags,
+                std::shared_ptr<File> file,
+                std::vector<Directory::Entry>&& dirents)
+    : file(file), flags(flags), dirents(std::move(dirents)) {}
+
+  [[nodiscard]] static int create(std::shared_ptr<File> file,
+                                  oflags_t flags,
+                                  std::shared_ptr<OpenFileState>& out);
 
   class Handle {
     std::shared_ptr<OpenFileState> openFileState;
@@ -55,8 +87,11 @@ public:
 
     std::shared_ptr<File>& getFile() { return openFileState->file; };
 
-    off_t getPosition() { return openFileState->position; };
+    off_t getPosition() const { return openFileState->position; };
     void setPosition(off_t pos) { openFileState->position = pos; };
+
+    oflags_t getFlags() const { return openFileState->flags; };
+    void setFlags(oflags_t flags) { openFileState->flags = flags; };
   };
 
   Handle locked() { return Handle(shared_from_this()); }
@@ -82,7 +117,14 @@ public:
       : fileTable(fileTable), lock(fileTable.mutex) {}
 
     std::shared_ptr<OpenFileState> getEntry(__wasi_fd_t fd);
-    void setEntry(__wasi_fd_t fd, std::shared_ptr<OpenFileState> openFile);
+
+    // Set the table slot at `fd` to the given file. If this overwrites the last
+    // reference to an OpenFileState for a data file in the table, return the
+    // file so it can be closed by the caller. Do not close the file directly in
+    // this method so it can be closed later while the FileTable lock is not
+    // held.
+    [[nodiscard]] std::shared_ptr<DataFile>
+    setEntry(__wasi_fd_t fd, std::shared_ptr<OpenFileState> openFile);
     __wasi_fd_t addEntry(std::shared_ptr<OpenFileState> openFileState);
   };
 
