@@ -11,31 +11,37 @@ import subprocess
 script_dir = os.path.dirname(os.path.abspath(__file__))
 root_dir = os.path.dirname(os.path.dirname(script_dir))
 
-sys.path.append(root_dir)
+sys.path.insert(0, root_dir)
 
-import emscripten
-from tools.settings import settings
+from tools import utils
 
 
 def check_structs(info):
-  for struct in info['structs'].keys():
-    key = 'C_STRUCTS.' + struct + '.'
+  for struct, values in info['structs'].items():
+    key = 'C_STRUCTS\\.' + struct + '\\.'
     # grep --quiet ruturns 0 when there is a match
     if subprocess.run(['git', 'grep', '--quiet', key], check=False).returncode != 0:
-      print(struct)
+      print(key)
+    else:
+      for value in values:
+        if value != '__size__':
+          key = 'C_STRUCTS\\.' + struct + '\\.' + value
+          # grep --quiet ruturns 0 when there is a match
+          if subprocess.run(['git', 'grep', '--quiet', key], check=False).returncode != 0:
+            print(key)
 
 
 def check_defines(info):
   for define in info['defines'].keys():
-    key = 'cDefine(.' + define + '.)'
+    key = r'cDefs\.' + define + r'\>'
     # grep --quiet ruturns 0 when there is a match
     if subprocess.run(['git', 'grep', '--quiet', key], check=False).returncode != 0:
       print(define)
 
 
 def main():
-  emscripten.generate_struct_info()
-  info = json.loads(open(settings.STRUCT_INFO).read())
+  json_file = utils.path_from_root('src/struct_info_generated.json')
+  info = json.loads(utils.read_file(json_file))
   check_structs(info)
   check_defines(info)
   return 0
